#!/usr/bin/env python
"""
subsection.py, module definition of Subsection class.
This defines a subsection of the presentation.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
from collections import OrderedDict
import re
# MaTiSSe.py modules
from ..config import __config__
from ..utils.source_editor import __source_editor__
from .regexs import  __regex_slide__
from .slide import Slide
# class definition
class Subsection(object):
  """
  Subsection is an object that handles a single subsection, its attributes and methods.

  Attributes
  ----------
  subsections_number : int
    global number of subsections (equals to the number of Subsection instances)
  """
  subsections_number = 0

  @classmethod
  def reset(cls):
    """Method resetting Subsection to initial values."""
    cls.subsections_number = 0
    return

  def __init__(self,raw_body='',title='',data=None,local_number=1):
    """
    Parameters
    ----------
    raw_body : str, optional
      string containing the body of the subsection in raw format
    title : str, optional
      subsection title
    data : OrderedDict object, optional
      subsection metadata
    local_number : int, optional
      subsection number in local-to-section numeration

    Attributes
    ----------
    raw_body : str
      subslide number in global numeration
    number : int
      subsection number in global numeration
    local_number : int
      subsection number in local-to-section numeration
    title : str
      subsection title
    data : OrderedDict object
      subsection metadata
    slides : list
      list of slides
    remainder : str
      remainder data that are not data slides
    """
    Subsection.subsections_number += 1
    self.raw_body     = raw_body
    self.number       = Subsection.subsections_number
    self.local_number = local_number
    self.title        = title
    self.data         = OrderedDict()
    if data:
      for key,val in data.items():
        self.data[key] = val
    self.data['subsectiontitle' ] = self.title
    self.data['subsectionnumber'] = str(self.number)
    self.slides   = None
    self.remainder = None
    return

  def get_remainder(self):
    """Method for getting the remainder of the source in case there are no data slides."""
    self.remainder = self.raw_body
    if __config__.verbose:
      message = ['\nAttention: found a bad usage of presentation sectioning!']
      message.append('\nThere are not data slides!')
      message.append('\nThe data:\n"""\n')
      message.append(self.remainder)
      message.append('"""\nis placed without "### slide" section into the current section/subsection!')
      print(''.join(message))
    return

  def get_slides(self,theme):
    """Method for getting the slides contained into the subsection.

    Parameters
    ----------
    theme : Theme object
    section_number : int
      current section number
    """
    slides = []
    self.slides = []
    purged_source = __source_editor__.purge_codes(self.raw_body)
    for match in re.finditer(__regex_slide__,purged_source):
      slides.append([match.group('expr'),match.start(),match.end()])
    if __config__.toc_at_subsec_beginning:
      slides = [['Table of Contents',__config__.toc_at_subsec_beginning]] + slides
    for sdn,slide in enumerate(slides):
      if len(slide)==2:
        raw_body = '$toc('+str(slide[1])+')'
      else:
        if sdn < len(slides)-1:
          raw_body = self.raw_body[slide[2]+1:slides[sdn+1][1]]
        else:
          raw_body = self.raw_body[slide[2]+1:]
      first_of_sec = None
      first_of_subsec = None
      if sdn == 0:
        first_of_subsec = self.number
        if self.local_number == 1:
          first_of_sec = int(self.data['sectionnumber'])
      data = OrderedDict()
      data['first_of_sec'] = first_of_sec
      data['first_of_subsec'] = first_of_subsec
      data.update(self.data)
      self.slides.append(Slide(raw_body = raw_body, title = slide[0], data = data, theme = theme, local_number = sdn + 1))
    return



#!/usr/bin/env python
"""
titlepage.py, module definition of Titlepage class.
This defines the title page the presentation being designed as an helper
object for writing a slide with a style very different from others.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# MaTiSSe.py modules
from ..theme.theme import Theme
from ..utils.source_editor import __source_editor__ as seditor
from .regexs import __regex_titlepage__ as re_titlepage
from .regexs import __regex_endtitlepage__ as re_endtitlepage
from .slide import Slide
# class definition
class Titlepage(Slide):
  """
  Object for handling the title page, its attributes and methods.
  """
  def __init__(self,plain=False,source=None):
    """
    Parameters
    ----------
    plain : bool, optional
      flag for resetting the slide style to plain
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    found : bool
      flag for checking is title page was found
    plain : bool
      flag for resetting the slide style to plain
    """
    self.found = False
    self.plain = plain
    if source:
      self.get(source=source)
    return

  def __str__(self):
    string = ''
    if self.found:
      string = self.raw_body
    return string

  def get(self,source):
    """Method for getting the title page contained into the source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without titlepage
    """
    purged_source = seditor.purge_codes(source)
    match = re.search(re_titlepage,purged_source)
    if match:
      self.found = True
      super(Titlepage,self).__init__(title='titlepage')
      if match.group('plain') and str(match.group('plain')).lower() == 'plain':
        self.plain = True
        self.overtheme = Theme(set_all_custom=True)
      endmatch = re.search(re_endtitlepage,source[match.end():])
      if endmatch:
        end = endmatch.start()+match.end()
        self.raw_body = source[match.end():end]
        source = source[0:match.start()]+source[end:]
      else:
        self.raw_body = source[match.end():]
        source = source[0:match.start()]
    return source



#!/usr/bin/env python
"""
utils.py, module definition of MaTiSSe.py util functions.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import os
from shutil import copyfile, copytree, rmtree
# MaTiSSe.py modules
from ..config import __config__
# global variables
# regular expressions
__expr__ = r"(?P<expr>.*)"


def make_output_tree(output):
  """
  Function for creating output tree and copy MaTiSSe.py assets.
  """
  # checking output directory
  if not os.path.exists(output):
    os.makedirs(output)
  # creating css directory
  if not os.path.exists(output + 'css'):
    os.makedirs(output + 'css')
  # normalize.css
  css = os.path.join(os.path.dirname(__file__), 'css/normalize.css')
  copyfile(css, output + 'css/normalize.css')
  # creating jscript directory
  if not os.path.exists(output + 'js'):
    os.makedirs(output + 'js')
  # MathJax engine
  if not __config__.online_mathjax:
    if os.path.exists(output + 'js/MathJax'):
      rmtree(output + 'js/MathJax')
    jscript = os.path.join(os.path.dirname(__file__), 'js/MathJax')
    copytree(jscript, output + 'js/MathJax')
  # highlight.js
  if __config__.highlight:
    if os.path.exists(output + 'js/highlight'):
      rmtree(output + 'js/highlight')
    jscript = os.path.join(os.path.dirname(__file__), 'js/highlight')
    copytree(jscript, output + 'js/highlight')
  # countDown.js
  jscript = os.path.join(os.path.dirname(__file__), 'js/countDown.js')
  copyfile(jscript, output + 'js/countDown.js')
  # impress.js
  jscript = os.path.join(os.path.dirname(__file__), 'js/impress/impress.js')
  copyfile(jscript, output + 'js/impress.js')
  return



#!/usr/bin/env python
"""
theme_element.py, module definition of ThemeElement class.
This is a base class for building the presentation Theme.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import ast
import re
# MaTiSSe.py modules
from ..data.data import Data
from ..utils.source_editor import __source_editor__
# class definition
class ThemeElement(object):
  """
  Object for handling a theme element.
  """
  def __init__(self, data_tag, skip = None, special_keys = None, class_name = None):
    """
    Parameters
    ----------
    data_tag : str
      tag name enclosing the data
    skip : list, optional
      list of re.compile (compiled regular expression) of block to be skipped
    special_keys : list, optional
      list of special entries that must be handled differently from the standard ones
    class_name : str, optional
      name of the 'css class' corresponding to the theme element
    set_all_custom : bool, optional
      set all option as customized from users (useful for plain slides theme)

    Attributes
    ----------
    data_tag: str
      tag name enclosing the data
    data: Data object
    class_name : str
      name of the 'css class' corresponding to the theme element
    active : bool
      flag for inquiring if the current theme element is active or not
    """
    _skip = [__source_editor__.regex_overtheme,__source_editor__.regex_codeblock,__source_editor__.regex_codeinline]
    if skip:
      for skp in skip:
        _skip.append(skp)
    _special_keys = ['metadata','active']
    if special_keys:
      for key in special_keys:
        _special_keys.append(key)
    self.data_tag     = data_tag
    self.data         = Data(regex_start=r'[-]{3}'+data_tag,regex_end=r'[-]{3}end'+data_tag,
                             skip=_skip,special_keys=_special_keys)
    self.class_name   = class_name
    self.active       = True
    self.data.data['metadata'] = [[],  False]
    self.data.data['active'  ] = [True,False]
    return

  def __str__(self):
    string = ''
    if self.active:
      string = str(self.data)
    return string

  def get(self,source):
    """Method for getting raw data from source.
    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    >>> elem = ThemeElement('header')
    >>> source = '---header option1 = val1 ---endheader'
    >>> elem.get(source)
    >>> elem.data.data['option1'][0]
    'val1'
    """
    self.data.get(source)
    return

  def set_from(self,other):
    """Method for setting theme using data of other (theme element).

    Parameters
    ----------
    other : ThemeElement object

    >>> elem1 = ThemeElement('header')
    >>> elem1.data.data['option1'] = ['unset',False]
    >>> elem2 = ThemeElement('header')
    >>> source = '---header option1 = val1 ---endheader'
    >>> elem2.get(source)
    >>> elem1.set_from(elem2)
    >>> elem1.data.data['option1'][0]
    'val1'
    """
    if other.active:
      for key,val in self.data.data.items():
        if not val[1]:
          if key in other.data.data:
            self.data.data[key] = other.data.data[key]
      self.check_specials()
    return

  def set_all_custom(self):
    """Method for setting all data as customized by user (useful for plain slides theme).

    >>> elem = ThemeElement('header')
    >>> elem.data.data['option1'] = ['unset',False]
    >>> elem.set_all_custom()
    >>> elem.data.data['option1'][1]
    True
    """
    self.data.set_all_custom()
    return

  def check_specials(self):
    """
    Method for checking special data entries.
    All theme elements have the following special entries:
    1. metadata
    2. active
    Other particular special entries must be handled into the subclass method.

    >>> elem = ThemeElement('header')
    >>> source = '---header metadata = ["slidetitle"] \\n active = False---endheader'
    >>> elem.get(source)
    >>> eval(elem.data.data['metadata'][0])[0]
    'slidetitle'
    """
    for key,val in self.data.data.items():
      if val[1]:
        if key == 'metadata':
          self.data.data[key] = [ast.literal_eval(str(val[0])),True]
        elif key == 'active':
          self.active = ast.literal_eval(str(val[0]))
          self.data.data[key] = [self.active,True]
    return

  def get_css(self,only_custom=False,as_list=False):
    """
    Method for getting css from data.

    Parameters
    ----------
    only_custom : bool, optional
      consider only the (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the element if as_list = False
    list
      a list of one string containing the css code of the element if as_list = True

    >>> elem = ThemeElement('header',class_name='header')
    >>> source = '---header height = 10% ---endheader'
    >>> elem.get(source)
    >>> elem.get_css(only_custom=True)
    '\\n.header {\\n  height: 10%;\\n}\\n'
    """
    if self.class_name:
      css = '\n.'+self.class_name+' {'+self.data.get_css(only_custom=only_custom)+'\n}\n'
    else:
      css = self.data.get_css(only_custom=only_custom)
    if as_list:
      return [css]
    else:
      return css

  def strip(self,source):
    """Method for striping theme element data from source.

    Parameters
    ----------
    source : str

    Returns
    -------
    str
      source without the theme element data

    >>> elem = ThemeElement('header')
    >>> source = 'other contents before---header height = 10% ---endheader,other contents after'
    >>> elem.get(source)
    >>> elem.data.data['height'][0]
    '10%'
    >>> elem.strip(source)
    'other contents before,other contents after'
    """
    return self.data.strip(source)

  @staticmethod
  def put_logo(doc,metadata,style):
    """Method for putting logo element.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    metadata : OrderedDict object
      dictionary of metadata
    style : str
      style applied to element
    """
    if style:
      doc.stag('img',src=metadata['logo'],alt=metadata['logo'],style=style)
    else:
      doc.stag('img',src=metadata['logo'],alt=metadata['logo'])
    return

  @staticmethod
  def put_timer(doc,metadata,style):
    """Method for putting timer element.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    metadata : OrderedDict object
      dictionary of metadata
    style : str
      style applied to element
    """
    with doc.tag('span',klass='timercontainer'):
      if style:
        doc.attr(style=style)
      with doc.tag('div',klass='countDown'):
        with doc.tag('div'):
          doc.attr(klass='timer')
        if style:
          if 'controls' in style:
            with doc.tag('div',klass='timercontrols'):
              with doc.tag('input',type='button'):
                doc.attr(klass='btn reset',onclick='resetCountdown('+metadata['max_time']+');',value=' &#10227; ',title='reset')
              with doc.tag('input',type='button'):
                doc.attr(klass='btn stop',onclick='stopCountdown();',value=' &#9724; ',title='pause')
              with doc.tag('input',type='button'):
                doc.attr(klass='btn start',onclick='startCountdown();',value=' &#9654; ',title='start')
    return

  @staticmethod
  def put_toc(doc,style,deep):
    """Method for putting TOC element.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    style : str
      style applied to element
    deep : int
      depth of Table of Contents
    """
    with doc.tag('div'):
      if style:
        doc.attr(('style',style))
      doc.asis('$toc('+str(deep)+')')
    return

  def put_metadata(self,doc,metadata):
    """Method for putting metadata elements into the current theme element.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    metadata : OrderedDict object
      dictionary of metadata
    """
    for element in self.data.data['metadata'][0]:
      elem = element
      style = None
      deep = 1
      if isinstance(element,list):
        elem = element[0]
        style = element[1]
        if len(element)>=3:
          deep = element[2]
      if elem == 'logo' and elem in metadata:
        self.put_logo(doc=doc,metadata=metadata,style=style)
      elif elem == 'timer' and 'max_time' in metadata:
        self.put_timer(doc=doc,metadata=metadata,style=style)
      elif elem == 'toc':
        self.put_toc(doc=doc,style=style,deep=deep)
      else:
        custom = re.search(r'\|custom\|',elem)
        with doc.tag('div'):
          style_txt = 'float:left;'
          if style:
            style_txt += style
          doc.attr(('style',style_txt))
          if custom:
            doc.asis(re.sub(r'\|custom\|','',elem))
          elif elem in metadata:
            if isinstance(metadata[elem],list):
              value = ' , '.join(metadata[elem])
            else:
              value = str(metadata[elem])
            doc.asis(value)
    return

  def to_html(self,doc,style=None,content=None,metadata=None):
    """
    Method for inserting element contents into html.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    style: str, optional
      particular element style
    padding: bool, optional
      activate internal element padded container
    content: str, optional
      content of the element as raw string
    metadata : OrderedDict object
      dictionary of metadata
    """
    if self.active:
      if content or metadata:
        with doc.tag('div'):
          if self.class_name:
            doc.attr(klass=self.class_name)
          if style:
            doc.attr(style=style)
          if content:
            doc.asis(content)
          if metadata:
            self.put_metadata(doc=doc,metadata=metadata)
    return



#!/usr/bin/env python
"""
MaTiSSe.py, Markdown To Impressive Scientific Slides
"""
__appname__ = "MaTiSSe.py"
__description__ = "MaTiSSe.py, Markdown To Impressive Scientific Slides"
__long_description__ = "MaTiSSe.py, Markdown To Impressive Scientific Slides. It is a very simple and stupid-to-use (KISS) presentation maker based on simple markdown syntax producing high quality first-class html/css presentation with great support for scientific contents."
__version__ = "0.1.0"
__author__ = "Stefano Zaghi"
__author_email__ = "stefano.zaghi@gmail.com"
__license__ = "GNU General Public License v3 (GPLv3)"
__url__ = "https://github.com/szaghi/MaTiSSe"
# modules loading
# standard library modules: these should be present in any recent python distribution
import argparse
import os
import sys
# MaTiSSe.py modules
from .config import __config__
from .presentation.presentation import Presentation
from .utils.source_editor import __source_editor__ as seditor
from .utils.utils import make_output_tree
# setting CLI
__cliparser__ = argparse.ArgumentParser(prog=__appname__, description='MaTiSSe.py, Markdown To Impressive Scientific Slides')
__cliparser__.add_argument('-v', '--version', action='version', help='Show version', version='%(prog)s ' + __version__)
__cliparser__.add_argument('-i', '--input', required=False, action='store', default=None, help='Input file name of markdown source to be parsed')
__cliparser__.add_argument('-o', '--output', required=False, action='store', default=None, help='Output directory name containing the presentation files')
__cliparser__.add_argument('-t', '--theme', required=False, action='store', default=None, help='Select a builtin theme for initializing a new sample presentation')
__cliparser__.add_argument('-hs', '--highlight-style', required=False, action='store', default='github.css', help='Select the highlight.js style (default github.css); select "disable" to disable highligth.js', metavar='STYLE.CSS')
__cliparser__.add_argument('-s', '--sample', required=False, action='store', default=None, help='Generate a new sample presentation as skeleton of your one')
__cliparser__.add_argument('--toc-at-sec-beginning', required=False, action='store', default=None, help='Insert Table of Contents at each section beginning (default no): to activate indicate the TOC depth', metavar='TOC-DEPTH')
__cliparser__.add_argument('--toc-at-subsec-beginning', required=False, action='store', default=None, help='Insert Table of Contents at each subsection beginning (default no): to activate indicate the TOC depth', metavar='TOC-DEPTH')
__cliparser__.add_argument('--print-preamble', required=False, action='store_true', default=None, help='Print the preamble data as parsed from source')
__cliparser__.add_argument('--print-css', required=False, action='store_true', default=None, help='Print the css as parsed from source (if done)')
__cliparser__.add_argument('--print-options', required=False, action='store_true', default=None, help='Print the available options for each presentation element')
__cliparser__.add_argument('--print-highlight-styles', required=False, action='store_true', default=None, help='Print the available highlight.js style (default github.css)')
__cliparser__.add_argument('--print-themes', required=False, action='store_true', default=None, help='Print the list of the builtin themes')
__cliparser__.add_argument('--verbose', required=False, action='store_true', default=False, help='More verbose printing messages (default no)')
__cliparser__.add_argument('--indented', required=False, action='store_true', default=False, help='Indent html output file (default no, may corrupt slide rendering)')
__cliparser__.add_argument('--online-MathJax', required=False, action='store_true', default=None, help='Use online rendering of LaTeX equations by means of online MathJax service; default use offline, local copy of MathJax engine')
__sample__ = """
# Section 1

## Subsection 1

### Slide 1

#### A H4 heading

Lorem ipsum dolor sit amet...

#### Math

$$
x=\frac{-b\pm\sqrt{b^2-4ac}}{2a}
$$

$note
$content{Just a note enviroment}
$endnote
"""


# functions
def pprint(presentation, cliargs):
  """Print presentation infos.

  Parameters
  ----------
  presentation : Presentation object
    presentation data
  cliargs : argparse parsed object
    command line arguments parsed
  """
  if cliargs.print_preamble:
    print(presentation)
  if cliargs.print_css:
    print(presentation.get_css())
  if cliargs.print_options:
    print(presentation.get_options())
  if cliargs.print_highlight_styles:
    print(__config__.str_highlight_styles())
  if cliargs.print_themes:
    print(__config__.str_themes())
  return


def main():
  """
  Main function.
  """
  cliargs = __cliparser__.parse_args()
  __config__.update(cliargs=cliargs)
  __config__.printf()
  if cliargs.input:
    if not os.path.exists(cliargs.input):
      sys.stderr.write('Error: input file "' + cliargs.input + '" not found!')
      sys.exit(1)
    else:
      with open(cliargs.input, 'r') as mdf:
        source = mdf.read()
      source = seditor.includes(source=source)
      if cliargs.output:
        output = cliargs.output
      else:
        output = os.path.splitext(os.path.basename(cliargs.input))[0]
      output = os.path.normpath(output) + "/"
      make_output_tree(output=output)
      if __config__.verbose:
        print('Parsing source ' + cliargs.input)
      presentation = Presentation(source=source)
      presentation.save(output=output)
      pprint(presentation=presentation, cliargs=cliargs)
  if cliargs.sample:
    source = __config__.put_theme(source=__sample__)
    with open(cliargs.sample, 'w') as sample:
      sample.write(source)
    source = seditor.includes(source=source)
    presentation = Presentation(source=source)
    output = os.path.splitext(os.path.basename(cliargs.sample))[0] + os.sep
    make_output_tree(output=output)
    presentation.save(output=output)
    pprint(presentation=presentation, cliargs=cliargs)
  else:
    presentation = None
    if cliargs.print_preamble or cliargs.print_css or cliargs.print_options:
      presentation = Presentation(defaults=True)
    pprint(presentation=presentation, cliargs=cliargs)

if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""
canvas.py, module definition of Canvas class.
This defines the theme of the canvas on top of which all slides are rendered.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from .theme_element import ThemeElement
# class definition
class Canvas(ThemeElement):
  """
  Object for handling the presentation theme canvas, its attributes and methods.
  """
  def __init__(self,source=None):
    super(Canvas,self).__init__(data_tag=r'theme_canvas')
    self.data.data['background'] = ['radial-gradient(rgb(240, 240, 240), rgb(190, 190, 190))',False]
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nCanvas']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\nbody {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css



#!/usr/bin/env python
"""
data.py, module definition of Data class.
This is the generic database holding presentation settings, options, etc...
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
from collections import OrderedDict
import copy
import re
# MaTiSSe.py modules
from ..utils.source_editor import __source_editor__
# class definition
class Data(object):
  """Object for handling data of presentation.

  The "data" dictionary is an ordered dictionary where the entries are divided into 2
  types:

  1. standard entries which can be safely converted to css format;
  2. special entries whose key must be inserted into the "special_keys" list and that
     handeld differently from standard data.

  Into the source the data must be defined with the following syntax:
  regex_start
  option_name1 = option_value1
  option_name1 = option_value1
  ...
  regex_end
  """
  def __init__(self, regex_start = '', regex_end = '', skip = None, special_keys = None):
    """
    Parameters
    ----------
    regex_start : str, optional
      regural expression starting the matching block of data
    regex_end : str, optional
      regural expression ending the matching block of data
    skip : list, optional
      list of re.compile (compiled regular expression) of block to be skipped
    special_keys : list, optional
      list of strings containing the special keys of data entries

    Attributes
    regex_start : str
      regural expression starting the matching block of data
    regex_end : str
      regural expression ending the matching block of data
    skip : list
      list of compiled regular expression of block to be skipped
    special_keys : list
      list of strings containing the special keys of data entries
    regex : re.compile
      compiler regular expression of the matching block
    data : OrderedDict object
      dictionary of data entries; constitute the main database of presentation elements
      options
    ----------
    """
    self.regex_start  = regex_start
    self.regex_end    = regex_end
    self.skip         = skip
    self.special_keys = special_keys
    self.regex        = re.compile(r"(?P<rdata>"+regex_start+r"(?P<opts>.*?)"+regex_end+")",re.DOTALL)
    self.data         = OrderedDict()

  def __str__(self):
    string = []
    if self.data:
      string = ['  '+str(k)+' : '+str(v)+'\n' for k,v in self.data.items()]
    return ''.join(string)

  def __copy__(self):
    newone = type(self)()
    newone.__dict__.update(self.__dict__)
    return newone

  def __deepcopy__(self,memo):
    newone = type(self).__new__(type(self))
    newone.__dict__.update(self.__dict__)
    newone.special_keys = copy.deepcopy(self.special_keys, memo)
    newone.data = copy.deepcopy(self.data, memo)
    return newone

  def count(self,source):
    """Method for computing the number of data definitions present into the source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    int
      number of data definitions

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1---endmydata---mydata option2 = val2---endmydata'
    >>> mydata.count(source)
    2
    """
    if self.skip:
      for skip in self.skip:
        source = __source_editor__.purge(regex=skip,source=source)
    number = len(re.findall(self.regex,source))
    return number

  def set_all_custom(self):
    """Method for setting all data as customized by user (useful for plain slides theme)."""
    for data in self.data:
      self.data[data][1] = True
    return

  def get(self,source):
    """Method for getting data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1 ---endmydata'
    >>> mydata.get(source)
    >>> mydata.data['option1'][0]
    'val1'
    """
    if self.skip:
      for skip in self.skip:
        source = __source_editor__.purge(regex=skip,source=source)
    matching = self.regex.search(source)
    if matching:
      #raw = matching.group('rdata')
      raw = matching.group('opts')
      raw = re.sub(r'\&\& *\n','',raw)
      raw = raw.split('\n')
      for i,rdata in enumerate(raw):
        if re.search(self.regex_start,rdata) or re.search(self.regex_end,rdata) or rdata == '':
          raw.pop(i)
        elif not re.search('=',rdata):
          raw.pop(i)
      for i,rdata in enumerate(raw):
        key = rdata.split('=')[0].strip()
        val = rdata.split('=')[1].strip()
        self.data[key] = [val,True]
    return

  def get_options(self):
    """Method for getting the available data options.

    Returns
    -------
    str
      string with option_names = values pairs (without True/False custom tag)

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1 ---endmydata'
    >>> mydata.get(source)
    >>> mydata.get_options()
    '\\noption1 = val1'
    """
    string = []
    for key,val in self.data.items():
      string.append('\n'+key+' = '+str(val[0]))
    return ''.join(string)

  def get_css(self,only_custom=False):
    """Method for getting css from data.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data

    Returns
    -------
    str
      sting containing the css style options based on the data options

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1 ---endmydata'
    >>> mydata.get(source)
    >>> mydata.get_css()
    '\\n  option1: val1;'
    """
    css = ''
    for key,val in self.data.items():
      special = False
      if self.special_keys and key in self.special_keys:
        special = True
      if not special:
        if only_custom:
          if val[1]:
            css += '\n  '+key+': '+val[0]+';'
        else:
          css += '\n  '+key+': '+val[0]+';'
    return css

  def get_custom(self,chk_specials=False):
    """Method returning only the data that have been set by users (customized) overriding default values.

    Parameters
    ----------
    chk_specials : bool, optional
      if activated handle special entries differently from standard ones

    Returns
    -------
    list
      list of only customized options

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1 ---endmydata'
    >>> mydata.get(source)
    >>> str(mydata.get_custom())
    "  option1 : ['val1', True]\\n"
    """
    custom = copy.deepcopy(self)
    for key,val in custom.data.items():
      special = False
      if self.special_keys and key in self.special_keys:
        special = True
      if chk_specials and special:
        continue
      else:
        if not val[1]:
          custom.data.pop(key,None)
    return custom

  def merge(self,otherdata):
    """Method merging otherdata with self. Only data that are not customized is updated with the otherdata data
    if they have been customized.

    Parameters
    ----------
    otherdata : Data object
      other data to be merged with self

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = '---mydata option1 = val1 ---endmydata'
    >>> mydata.get(source)
    >>> mydata.data['option2'] = ['unset',False]
    >>> otherdata = Data('---mydata','---endmydata')
    >>> source = '---mydata option2 = val2 ---endmydata'
    >>> otherdata.get(source)
    >>> mydata.merge(otherdata)
    >>> mydata.data['option2'][0]
    'val2'
    """
    for key,val in self.data.items():
      if not val[1]:
        if key in otherdata.data and otherdata.data[key][1]:
          self.data[key] = otherdata.data[key]
    return

  def strip(self,source):
    """Method for striping raw data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without the raw data

    >>> mydata = Data('---mydata','---endmydata')
    >>> source = 'other contents before---mydata option1 = val1 ---endmydata,other contents after'
    >>> mydata.strip(source)
    'other contents before,other contents after'
    """
    if self.skip:
      strip_source = source
      pattern = '|'.join([ skip.pattern for skip in self.skip ])
      regex = re.compile(pattern+r"|(?P<strip>"+self.regex.pattern+r")",re.DOTALL)
      matches = []
      for match in re.finditer(regex,strip_source):
        if match.group('strip'):
          matches.append([match.start(),match.end()])
      if len(matches)>0:
        strip = ''
        for mtc,match in enumerate(matches):
          if mtc == 0:
            start = 0
          else:
            start = matches[mtc-1][1]+1
          if match[0]!=start:
            strip += strip_source[start:match[0]]
        if matches[-1][1]<len(strip_source)+1:
          strip += strip_source[matches[-1][1]:]
        strip_source = strip
    else:
      strip_source = re.sub(self.regex,'',source)
    return strip_source.strip()



#!/usr/bin/env python
"""
position.py, module definition of Position class.
This defines the position features of the slide element.
"""
# class definition
class Position(object):
  """
  Object for handling slide position into the canvas.
  """
  def __init__(self,pos=None):
    """
    Attributes
    ----------
    position : list
      list of 3 int containing x,y,z position values
    rotation : list
      list of 3 int containing x,y,z rotation values
    scale : int
      scaling factor
    offset : int
      distance between consecutive slide in percent (%)
    """
    if pos:
      self.position = pos
    else:
      self.position = [0,0,0] # x,y,z
    self.rotation = [0,0,0] # around x,y,z
    self.scale = 1
    self.offset = 1
    return

  def __str__(self):
    string = ['\nPosition (x,y,z) = '+','.join([str(p) for p in self.position])]
    string.append('\nRotation (x,y,z) = '+','.join([str(r) for r in self.rotation]))
    string.append('\nScale factor = '+str(self.scale))
    return ''.join(string)

  @staticmethod
  def get_scale(theme):
    """Method for computing the current slide scale factor.

    Parameters
    ----------
    theme : Theme object
      current global theme

    Returns
    -------
    int
      scaling factor

    >>> from .slide import Slide
    >>> source = '---theme_slide_global data-scale = 3 ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos = Position()
    >>> pos.get_scale(theme=theme)
    3
    """
    return int(theme.data.data['data-scale'][0])

  @staticmethod
  def get_offset(theme):
    """Method for computing the current slide offset factor.

    Parameters
    ----------
    theme : Theme object
      current global theme

    Returns
    -------
    int
      offset factor

    >>> from .slide import Slide
    >>> source = '---theme_slide_global data-offset = 200 ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos = Position()
    >>> pos.get_offset(theme=theme)
    200
    """
    return int(theme.data.data['data-offset'][0])

  @staticmethod
  def get_rotation(theme):
    """Method for computing the current slide rotation.

    Parameters
    ----------
    theme : Theme object
      current global theme

    Returns
    -------
    list
      list of 3 int containing x,y,z rotation values

    >>> from .slide import Slide
    >>> source = '---theme_slide_global data-rotate-x = 45 \\n data-rotate-y = 65 \\n data-rotate-z = 85 ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos = Position()
    >>> pos.get_rotation(theme=theme)
    [45, 65, 85]
    """
    rot = int(theme.data.data['data-rotate'][0])
    rot_x = int(theme.data.data['data-rotate-x'][0])
    rot_y = int(theme.data.data['data-rotate-y'][0])
    rot_z = int(theme.data.data['data-rotate-z'][0])
    if rot_z != 0:
      rot = max(rot,rot_z)
    return [rot_x,rot_y,rot]

  def get_position(self,theme,scale,offset):
    """Method for computing the current slide position.

    Parameters
    ----------
    theme : Theme object
      current global theme
    scale : int
      factor scaling of previous slide
    offset : int
      offset factor

    Returns
    -------
    list
      list of 3 int containing x,y,z position values

    >>> from .slide import Slide
    >>> source = '---theme_slide_global slide-transition = absolute \\n data-x = 45 \\n data-y = 65 \\n data-z = 85 ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos = Position()
    >>> pos.get_position(theme=theme,scale=1,offset=1)
    [45, 65, 85]
    >>> source = '---theme_slide_global width = 1 \\n slide-transition = Horizontal ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [2.0, 0, 0]
    >>> source = '---theme_slide_global width = 1 \\n slide-transition = -Horizontal ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [-2.0, 0, 0]
    >>> source = '---theme_slide_global height = 1 \\n slide-transition = Vertical ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [0, 2.0, 0]
    >>> source = '---theme_slide_global height = 1 \\n slide-transition = -Vertical ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [0, -2.0, 0]
    >>> source = '---theme_slide_global width = 1 \\n height = 1 \\n slide-transition = Diagonal ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [2.0, 2.0, 0]
    >>> source = '---theme_slide_global width = 1 \\n height = 1 \\n slide-transition = -Diagonal ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [-2.0, -2.0, 0]
    >>> source = '---theme_slide_global width = 1 \\n height = 1 \\n slide-transition = Diagonal-X ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [-2.0, 2.0, 0]
    >>> source = '---theme_slide_global width = 1 \\n height = 1 \\n slide-transition = Diagonal-Y ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos.get_position(theme=theme,scale=1,offset=100)
    [2.0, -2.0, 0]
    """
    pos_x = int(theme.data.data['data-x'][0])
    pos_y = int(theme.data.data['data-y'][0])
    pos_z = int(theme.data.data['data-z'][0])
    slide_width = int(theme.data.data['width'][0].strip('px'))
    slide_height = int(theme.data.data['height'][0].strip('px'))
    slide_transition = theme.data.data['slide-transition'][0]
    if slide_transition.lower() == 'absolute':
      pass
    elif slide_transition.lower() == 'svgpath':
      pos_x = self.position[0]
      pos_y = self.position[1]
      pos_z = self.position[2]
    elif slide_transition.lower() == 'horizontal':
      pos_x = self.position[0] + slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1]
    elif slide_transition.lower() == '-horizontal':
      pos_x = self.position[0] - slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1]
    elif slide_transition.lower() == 'vertical':
      pos_x = self.position[0]
      pos_y = self.position[1] + slide_height*(max(self.scale,scale)+offset/100.0)
    elif slide_transition.lower() == '-vertical':
      pos_x = self.position[0]
      pos_y = self.position[1] - slide_height*(max(self.scale,scale)+offset/100.0)
    elif slide_transition.lower() == 'diagonal':
      pos_x = self.position[0] + slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1] + slide_height*(max(self.scale,scale)+offset/100.0)
    elif slide_transition.lower() == '-diagonal':
      pos_x = self.position[0] - slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1] - slide_height*(max(self.scale,scale)+offset/100.0)
    elif slide_transition.lower() == 'diagonal-x':
      pos_x = self.position[0] - slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1] + slide_height*(max(self.scale,scale)+offset/100.0)
    elif slide_transition.lower() == 'diagonal-y':
      pos_x = self.position[0] + slide_width*(max(self.scale,scale)+offset/100.0)
      pos_y = self.position[1] - slide_height*(max(self.scale,scale)+offset/100.0)
    return [pos_x,pos_y,pos_z]

  def set_position(self,theme):
    """Method for setting positioning data inquiring the (base) theme and the eventually present overriding one.

    Parameters
    ----------
    theme : Theme object
      current theme

    >>> from .slide import Slide
    >>> source = '---theme_slide_global slide-transition = absolute \\n data-x = 45 \\n data-y = 65 \\n data-z = 85 ---endtheme_slide_global'
    >>> theme = Slide(source=source)
    >>> pos = Position()
    >>> pos.set_position(theme=theme)
    >>> pos.position
    [45, 65, 85]
    """
    scale = self.get_scale(theme=theme)
    offset = self.get_offset(theme=theme)

    self.rotation = self.get_rotation(theme=theme)

    self.position = self.get_position(theme=theme,scale=scale,offset=offset)

    self.scale = scale
    self.offset = offset
    return



#!/usr/bin/env python
"""
table.py, module definition of Table class.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# modules not in the standard library
from yattag import Doc
# MaTiSSe.py modules
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import illuminate_protected as illuminate
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
from ..utils.source_editor import tokenize as generic_tokenize
from .box import Box
from .theme_element import ThemeElement
# global variables
# regular expressions
__retable__ = re.compile(r"\$table(?P<env>.*?)\$endtable",re.DOTALL)
# classes definition
class Table(Box):
  """
  Object for handling table-box. It is a subclass of Box.

  The syntax is:

  $table
  $style[style_options]
  $caption[caption_options]{caption}
  $content[content_options]{content}
  $endtable

  Note that differently from Box class:
  1. the "content_type" and "caption_type" are automatically set to "table" and "Table" respectively; anyhow they can be still
     specified inside the $table/$endtable environment;
  2. no matter the order of $caption / $content statements, the caption is always placed above the content;

  Attributes
  ----------
  tables_number : int
    global number of tables (equals to the number of Table instances)
  theme: ThemeElement object
    global theme of Table boxes
  """
  tables_number = 0
  theme = ThemeElement(data_tag=r'theme_table')
  theme.data.data['style'  ] = [None,False]
  theme.data.data['caption'] = [None,False]
  theme.data.data['content'] = [None,False]

  @classmethod
  def reset(cls):
    """Method resetting Table to initial values."""
    cls.tables_number = 0
    cls.theme = ThemeElement(data_tag=r'theme_table')
    cls.theme.data.data['style'  ] = [None,False]
    cls.theme.data.data['caption'] = [None,False]
    cls.theme.data.data['content'] = [None,False]
    return

  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      number of table
    """
    super(Table,self).__init__(ctn_type='table')
    self.cap_type = 'Table'
    Table.tables_number += 1
    self.number = Table.tables_number
    if source:
      self.get(source=source)
    return

  @classmethod
  def get_theme(cls,source):
    """Method for getting theme definition table boxes.

    The syntax is:

    ---theme_table
    style   = style_options
    caption = caption_options
    content = content_options
    ---endtheme_table

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    cls.theme.get(source=source)
    return

  @classmethod
  def strip_theme(cls,source):
    """Method for striping theme data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without the themes data
    """
    return cls.theme.strip(source=source)

  def put_caption(self,doc):
    """Method for inserting caption into doc.

    Parameters
    ----------
    doc : yattag.Doc object
      yattag document where to put caption
    """
    if self.cap or self.cap_type:
      with doc.tag('div',klass='table caption'):
        if self.cap_options:
          doc.attr(style=self.cap_options)
        elif Table.theme.data.data['caption'][0]:
          doc.attr(style=Table.theme.data.data['caption'][0])
        doc.asis(self.caption_txt())
    return

  def to_html(self):
    """Method for inserting box to the html doc."""
    doc = Doc()
    with doc.tag('div',klass='table'):
      if self.style:
        doc.attr(style=self.style)
      elif Table.theme.data.data['style'][0]:
        doc.attr(style=Table.theme.data.data['style'][0])
      self.put_caption(doc=doc)
      with doc.tag('div',klass='table content'):
        if self.ctn_options:
          doc.attr(style=self.ctn_options)
        elif Table.theme.data.data['content'][0]:
          doc.attr(style=Table.theme.data.data['content'][0])
        doc.asis(seditor.md_convert(self.ctn))
    return doc.getvalue()

def tokenize(source):
  """Method for tokenizing source tagging tables environments.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  list
    list of tokens whose elements are ['type',source_part]; type is 'table' for note environments and
    'unknown' for anything else
  """
  return generic_tokenize(source=source,re_part=__retable__,name_part='table')

def parse(source):
  """Method for parsing source substituting figures with their own html equivalent.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  str
    source string parsed
  """
  protected, obfuscate_source = obfuscate(source = source)
  for match in re.finditer(__retable__,obfuscate_source):
    table = Table(source=illuminate(source=match.group('env'),protected_contents=protected))
    obfuscate_source = re.sub(__retable__,table.to_html(),obfuscate_source,1)
  return illuminate(source=obfuscate_source,protected_contents=protected)



#!/usr/bin/env python
"""
metadata.py, module definition of Metadata class.
This defines the presentation metadata object.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import ast
import re
# modules not in the standard library
from yattag import Doc
# MaTiSSe.py modules
from ..data.data import Data
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
from ..utils.source_editor import illuminate_protected as illuminate
# class definition
class Metadata(object):
  """
  Object for handling the presentation metadata.
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    data : Data object
      presentation metadata

    >>> source = '---metadata max_time = 60 ---endmetadata'
    >>> meta = Metadata(source)
    >>> meta.data.data['max_time'][0]
    '60'
    """
    _skip = [seditor.regex_codeblock,seditor.regex_codeinline]
    self.data = Data(regex_start='[-]{3}metadata',regex_end='[-]{3}endmetadata',skip=_skip,special_keys=['__all__'])
    self.data.data['title'              ] = ['',  False]
    self.data.data['subtitle'           ] = ['',  False]
    self.data.data['authors'            ] = [[],  False]
    self.data.data['authors_short'      ] = [[],  False]
    self.data.data['emails'             ] = [[],  False]
    self.data.data['affiliations'       ] = [[],  False]
    self.data.data['affiliations_short' ] = [[],  False]
    self.data.data['logo'               ] = ['',  False]
    self.data.data['location'           ] = ['',  False]
    self.data.data['location_short'     ] = ['',  False]
    self.data.data['date'               ] = ['',  False]
    self.data.data['conference'         ] = ['',  False]
    self.data.data['conference_short'   ] = ['',  False]
    self.data.data['session'            ] = ['',  False]
    self.data.data['session_short'      ] = ['',  False]
    self.data.data['max_time'           ] = ['25',False]
    self.data.data['total_slides_number'] = ['',  False]
    self.data.data['dirs_to_copy'       ] = [[],  False]
    self.data.data['toc'                ] = ['',  False]
    if source:
      self.get(source)
    return

  def __str__(self):
    string = 'Metadata\n'
    string += str(self.data)
    return string

  def get(self,source):
    """Method for getting the presentation metadata.

    Return the source without the metadata.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without metadata
    """
    self.data.get(source)
    for key,val in self.data.data.items():
      if val[1]:
        if (key == 'authors' or
            key == 'authors_short' or
            key == 'emails' or
            key == 'affiliations' or
            key == 'affiliations_short' or
            key == 'dirs_to_copy'):
          self.data.data[key] = [ast.literal_eval(str(val[0])),True]
    return self.data.strip(source)

  def get_options(self):
    """Method for getting the available data options.

    Returns
    -------
    str
      string with option_names = values pairs (without True/False custom tag)

    >>> meta = Metadata()
    >>> meta.get_options().split('\\n')[16]
    'max_time = 25'
    """
    string = ['Presentation metadata']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_value(self,metadata):
    """Method for getting metadata value (returned as string).

    Parameters
    ----------
    metadata : str
      metadata key

    Returns
    -------
    str
      metadata value

    >>> source = '---metadata authors = ["S. Zaghi","J. Doe"] ---endmetadata'
    >>> meta = Metadata(source)
    >>> meta.get_value(metadata='authors')
    'S. Zaghi and J. Doe'
    """
    if metadata in self.data.data:
      if metadata == 'authors' or metadata == 'authors_short':
        value = ' and '.join(self.data.data[metadata][0])
      elif metadata == 'emails' or metadata == 'affiliations' or metadata == 'affiliations_short' or metadata == 'dirs_to_copy':
        value = ', '.join(self.data.data[metadata][0])
      else:
        value = str(self.data.data[metadata][0])
    return value

  def put_logo(self,doc,style):
    """Method for putting logo element.

    Parameters
    ----------
    doc : yattag.Doc object
      the currently open yattag.Doc object
    metadata : OrderedDict object
      dictionary of metadata
    style : str
      style applied to element
    """
    with doc.tag('figure'):
      if style:
        doc.stag('img',src=self.get_value('logo'),alt=self.get_value('logo'),style=style)
      else:
        doc.stag('img',src=self.get_value('logo'),alt=self.get_value('logo'))
    return

  def to_html(self,metadata,style=None):
    """Method for producing and html string of selected metadata.

    Parameters
    ----------
    metadata : str
      metadata key
    style : str, optional
      css style of metadata tag

    Returns
    -------
    str
      html string containing the metadata

    >>> source = '---metadata authors = ["S. Zaghi","J. Doe"] ---endmetadata'
    >>> meta = Metadata(source)
    >>> meta.to_html(metadata='authors')
    '<span class="metadata">S. Zaghi and J. Doe</span>'
    """
    doc = Doc()
    if metadata == 'logo':
      self.put_logo(doc=doc,style=style)
    else:
      with doc.tag('span',klass='metadata'):
        if style:
          doc.attr(style=style)
        doc.asis(self.get_value(metadata))
    return doc.getvalue()

  def parse(self,source):
    """Method for parsing source and substituting metadata placeholders with theis html equivalent.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source string parsed

    >>> source = '---metadata authors = ["S. Zaghi","J. Doe"] ---endmetadata $authors'
    >>> meta = Metadata(source)
    >>> meta.parse(source)
    '---metadata authors = ["S. Zaghi","J. Doe"] ---endmetadata <span class="metadata">S. Zaghi and J. Doe</span>'
    """
    protected, obfuscate_source = obfuscate(source = source)
    for meta in self.data.data:
      if meta !='toc':
        regex = re.compile(r"\$"+meta+r"(\[(?P<style>.*?)\])*",re.DOTALL)
        for match in re.finditer(regex,obfuscate_source):
          style = None
          if match.group('style'):
            style = str(match.group('style'))
          obfuscate_source = re.sub(regex,self.to_html(metadata=meta,style=style),obfuscate_source,1)
    return illuminate(source=obfuscate_source,protected_contents=protected)



#!/usr/bin/env python
"""
heading.py, module definition of Heading class.
This defines the theme of the headings.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from .theme_element import ThemeElement
# default size of each heading of the standard 6 headings (h1, h2, etc...)
__sizes__ = ['220%','200%','180%','160%','140%','120%']
# class definition
class Heading(ThemeElement):
  """
  Object for handling the presentation theme headings, its attributes and methods.
  """
  def __init__(self,number,source=None):
    self.number = number
    super(Heading,self).__init__(data_tag=r'theme_heading_'+str(self.number))
    #self.data.data['margin'         ] = ['inherit',                False]
    #self.data.data['padding'        ] = ['inherit',                False]
    #self.data.data['background'     ] = ['inherit',                False]
    #self.data.data['color'          ] = ['inherit',                False]
    #self.data.data['font'           ] = ['inherit',                False]
    #self.data.data['font-weight'    ] = ['inherit',                False]
    self.data.data['font-size'      ] = [ __sizes__[self.number-1],False]
    #self.data.data['font-family'    ] = ['inherit',                False]
    #self.data.data['text-decoration'] = ['inherit',                False]
    #self.data.data['text-transform' ] = ['inherit',                False]
    #self.data.data['text-shadow'    ] = ['inherit',                False]
    #self.data.data['letter-spacing' ] = ['inherit',                False]
    #self.data.data['line-height'    ] = ['inherit',                False]
    #self.data.data['border'         ] = ['inherit',                False]
    #self.data.data['border-radius'  ] = ['inherit',                False]
    #self.data.data['box-shadow'     ] = ['inherit',                False]
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nh'+str(self.number)]
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\nh"+str(self.number)+" {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css



#!/usr/bin/env python
"""
slide.py, module definition of Slide class.
This defines the theme of the slide element.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from .content import Content
from .footer import Footer
from .header import Header
from .sidebar import Sidebar
from ..theme_element import ThemeElement
from ...utils.source_editor import __source_editor__ as seditor
# class definition
class Slide(ThemeElement):
  """
  Object for handling the presentation theme slide, its attributes and methods.
  """
  def __init__(self,source=None,defaults=False):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    defaults : bool, optional
      flag for activatin the creation of a presentation istance
      having one of each element available with the default
      settings

    Attributes
    ----------
    content : Content object
      the main content element of the slide
    headers : list
      list of Header objects for handling the slide headers
    footers : list
      list of Footer objects for handling the slide footers
    sidebars : list
      list of Sidebar objects for handling the slide sidebars

    >>> sld1 = Slide()
    >>> sld1.data.data['width'][0]
    '900px'
    >>> sld2 = Slide(defaults=True)
    >>> sld2.has_header()
    (True, 1)
    """
    self.content  = Content()
    self.headers  = {}
    self.footers  = {}
    self.sidebars = {}
    super(Slide,self).__init__(data_tag=r'theme_slide_global',
                               special_keys=['slide-transition',
                                             'data-scale',
                                             'data-rotate','data-rotate-x','data-rotate-y','data-rotate-z',
                                             'data-x','data-y','data-z',
                                             'data-offset'])
    self.data.data['width'           ] = ['900px',                       False]
    self.data.data['height'          ] = ['700px',                       False]
    self.data.data['font-size'       ] = ['100%',                        False]
    self.data.data['font-family'     ] = ['Open Sans, Arial, sans-serif',False]
    self.data.data['slide-transition'] = ['horizontal',                  False]
    self.data.data['data-scale'      ] = ['1',                           False]
    self.data.data['data-rotate'     ] = ['0',                           False]
    self.data.data['data-rotate-x'   ] = ['0',                           False]
    self.data.data['data-rotate-y'   ] = ['0',                           False]
    self.data.data['data-rotate-z'   ] = ['0',                           False]
    self.data.data['data-x'          ] = ['0',                           False]
    self.data.data['data-y'          ] = ['0',                           False]
    self.data.data['data-z'          ] = ['0',                           False]
    self.data.data['data-offset'     ] = ['1',                           False]
    if source:
      self.get(source)
    elif defaults:
      self.headers['slide-header_1'] = Header(number=1)
      self.footers['slide-footer_1'] = Footer(number=1)
      self.sidebars['slide-sidebar_1'] = Sidebar(number=1)
    return

  def __str__(self):
    """
    >>> source =  '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> source += '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> sld1 = Slide(source=source)
    >>> str(sld1).split('\\n')[4]
    "  width : ['900px', False]"
    """
    string = '\n  Global slide\n'
    string += super(Slide, self).__str__()
    if self.has_header():
      for header in self.headers.values():
        string += '\n  Header n.'+str(header.number)+'\n'+str(header)
    if self.has_footer():
      for footer in self.footers.values():
        string += '\n  Footer n.'+str(footer.number)+'\n'+str(footer)
    if self.has_sidebar():
      for sidebar in self.sidebars.values():
        string += '\n  Sidebar n.'+str(sidebar.number)+'\n'+str(sidebar)
    string += '\n  Content\n'+str(self.content)
    return string

  def has_header(self,source=None):
    """Method for inquiring the presence of headers.

    Parameters
    ----------
    source : str

    Returns
    -------
    bool
      True is Slide has at least one header, False otherwise
    int
      number of headers (0 if Slide has not header at all)

    >>> source = '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1 ---theme_slide_header_2 height = 15% ---endtheme_slide_header_2'
    >>> sld1 = Slide(source=source)
    >>> sld1.has_header()
    (True, 2)
    >>> sld2 = Slide()
    >>> sld2.has_header(source=source)
    (True, 2)
    """
    if source:
      purged_source = seditor.purge_overtheme(source=source)
      purged_source = seditor.purge_codeblocks(source=purged_source)
      number = int(purged_source.count('theme_slide_header')/2)
    else:
      number = len(self.headers)
    return number>0,number

  def has_footer(self,source=None):
    """Method for inquiring the presence of footers.

    Parameters
    ----------
    source : str

    Returns
    -------
    bool
      True is Slide has at least one footer, False otherwise
    int
      number of footers (0 if Slide has not footer at all)

    >>> source = '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1 ---theme_slide_footer_2 height = 15% ---endtheme_slide_footer_2'
    >>> sld1 = Slide(source=source)
    >>> sld1.has_footer()
    (True, 2)
    >>> sld2 = Slide()
    >>> sld2.has_footer(source=source)
    (True, 2)
    """
    if source:
      purged_source = seditor.purge_overtheme(source=source)
      purged_source = seditor.purge_codeblocks(source=purged_source)
      number = int(purged_source.count('theme_slide_footer')/2)
    else:
      number = len(self.footers)
    return number>0,number

  def has_sidebar(self,source=None):
    """Method for inquiring the presence of sidebars.

    Parameters
    ----------
    source : str

    Returns
    -------
    bool
      True is Slide has at least one sidebar, False otherwise
    int
      number of sidebars (0 if Slide has not sidebar at all)

    >>> source = '---theme_slide_sidebar_1 height = 10% ---endtheme_slide_sidebar_1 ---theme_slide_sidebar_2 height = 15% ---endtheme_slide_sidebar_2'
    >>> sld1 = Slide(source=source)
    >>> sld1.has_sidebar()
    (True, 2)
    >>> sld2 = Slide()
    >>> sld2.has_sidebar(source=source)
    (True, 2)
    """
    if source:
      purged_source = seditor.purge_overtheme(source=source)
      purged_source = seditor.purge_codeblocks(source=purged_source)
      number = int(purged_source.count('theme_slide_sidebar')/2)
    else:
      number = len(self.sidebars)
    return number>0,number

  def loop_over_headers(self):
    """Method for looping over headers dictionary returning each header in
    oredered numeration.

    Returns
    -------
    Header object
    """
    has, number = self.has_header()
    if has:
      for hdr in range(number):
        yield self.headers['slide-header_'+str(hdr+1)]

  def loop_over_footers(self):
    """Method for looping over footers dictionary returning each footer in
    oredered numeration.

    Returns
    -------
    Footer object
    """
    has, number = self.has_footer()
    if has:
      for ftr in range(number):
        yield self.footers['slide-footer_'+str(ftr+1)]

  def loop_over_sidebars(self):
    """Method for looping over sidebars dictionary returning each sidebar in
    oredered numeration.

    Returns
    -------
    Sidebar object
    """
    has, number = self.has_sidebar()
    if has:
      for sbr in range(number):
        yield self.sidebars['slide-sidebar_'+str(sbr+1)]

  def get(self,source):
    """Method for getting data values from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(Slide, self).get(source)
    self.check_specials()
    self.content.get(source)
    has,number = self.has_header(source)
    if has:
      for hdr in range(number):
        self.headers['slide-header_'+str(hdr+1)] = Header(number=hdr+1,source=source)
    has,number = self.has_footer(source)
    if has:
      for ftr in range(number):
        self.footers['slide-footer_'+str(ftr+1)] = Footer(number=ftr+1,source=source)
    has,number = self.has_sidebar(source)
    if has:
      for sbr in range(number):
        self.sidebars['slide-sidebar_'+str(sbr+1)] = Sidebar(number=sbr+1,source=source)
    self.adjust_dims()
    return

  def set_from(self,other):
    """Method for setting theme slide using data of other (theme slide element).

    Parameters
    ----------
    other : ThemeSlide object

    >>> source =  '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> source += '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> sld1 = Slide(source=source)
    >>> sld1.headers['slide-header_1'].data.data['background'] = ['white',False]
    >>> source =  '---theme_slide_header_1 background = red ---endtheme_slide_header_1'
    >>> source += '---theme_slide_header_2 background = red ---endtheme_slide_header_2'
    >>> source += '---theme_slide_footer_1 background = green ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_footer_2 background = green ---endtheme_slide_footer_2'
    >>> source += '---theme_slide_sidebar_1 background = blue ---endtheme_slide_sidebar_1'
    >>> source += '---theme_slide_sidebar_2 background = blue ---endtheme_slide_sidebar_2'
    >>> sld2 = Slide(source=source)
    >>> sld1.set_from(sld2)
    >>> sld1.headers['slide-header_1'].data.data['background'][0]
    'red'
    """

    self.content.set_from(other=other.content)

    if other.has_header():
      for header in other.headers:
        if header in self.headers:
          self.headers[header].set_from(other=other.headers[header])
        else:
          self.headers[header] = Header(number=other.headers[header].number)
          self.headers[header].set_from(other=other.headers[header])

    if other.has_footer():
      for footer in other.footers:
        if footer in self.footers:
          self.footers[footer].set_from(other=other.footers[footer])
        else:
          self.footers[footer] = Footer(number=other.footers[footer].number)
          self.footers[footer].set_from(other=other.footers[footer])

    if other.has_sidebar():
      for sidebar in other.sidebars:
        if sidebar in self.sidebars:
          self.sidebars[sidebar].set_from(other=other.sidebars[sidebar])
        else:
          self.sidebars[sidebar] = Sidebar(number=other.sidebars[sidebar].number)
          self.sidebars[sidebar].set_from(other=other.sidebars[sidebar])

    self.check_specials()
    return

  def set_all_custom(self):
    """Method for setting all data as customized by user (useful for plain slides theme).

    >>> sld1 = Slide(defaults=True)
    >>> sld1.set_all_custom()
    >>> sld1.data.data['width'][1]
    True
    """
    self.data.set_all_custom()
    self.content.set_all_custom()
    for hds in self.headers:
      self.headers[hds].set_all_custom()
    for ftr in self.footers:
      self.footers[ftr].set_all_custom()
    for sbr in self.sidebars:
      self.sidebars[sbr].set_all_custom()
    return

  #def update(self,source):
  #  """Method for updating data from source without creating new data.

  #  Parameters
  #  ----------
  #  source : str
  #    string (as single stream) containing the source
  #  """
  #  super(Slide,self).get(source)
  #  self.content.get(source)
  #  if self.has_header():
  #    for hdr in self.headers.values():
  #      hdr.get(source)
  #  if self.has_footer():
  #    for ftr in self.footers.values():
  #      ftr.get(source)
  #  if self.has_sidebar():
  #    for sbr in self.sidebars.values():
  #      sbr.get(source)
  #  self.adjust_dims()
  #  return

  def check_specials(self):
    """Method for checking specials data entries.

    This theme element has the following special entries:
    1. slide-transition
    2. data-scale
    3. data-rotate
    4. data-rotate-x
    5. data-rotate-y
    6. data-rotate-z
    7. data-x
    8. data-y
    9. data-z
    10. data-offset

    The check_specials method of other contained elements is called.
    """
    for key,val in self.data.data.items():
      if val[1]:
        if (key == 'slide-transition' or
            key == 'data-scale' or
            key == 'data-rotate' or key == 'data-rotate-x' or key == 'data-rotate-y' or key == 'data-rotate-z' or
            key == 'data-x' or key == 'data-y' or key == 'data-z' or
            key == 'data-offset'):
          self.data.data[key] = [val[0],True]
    self.content.check_specials()
    if self.has_header():
      for hdr in self.headers.values():
        hdr.check_specials()
    if self.has_footer():
      for ftr in self.footers.values():
        ftr.check_specials()
    if self.has_sidebar():
      for sbr in self.sidebars.values():
        sbr.check_specials()
    return

  def adjust_dims(self):
    """Method for adjusting dimensions of slide elements accordingly to the settings of other elements of the slide theme.
    """
    self.content.adjust_dims(headers=self.headers,footers=self.footers,sidebars=self.sidebars)
    if self.has_sidebar():
      for sidebar in self.sidebars.values():
        if sidebar.active:
          sidebar.adjust_dims(headers=self.headers,footers=self.footers)
    return

  def get_custom(self,chk_specials=False):
    """Method returning only the data that have been set by users (customized) overriding default values.

    Parameters
    ----------
    chk_specials : bool, optional
      if activated handle special entries differently from standard ones

    Returns
    -------
    dict
      dictionary of customized data

    >>> sld1 = Slide(defaults=True)
    >>> sld1.headers['slide-header_1'].data.data['background'] = ['white',True]
    >>> sld1.get_custom()['slide-header_1'].data['background'][0]
    'white'
    """
    custom = {}
    custom['slide-content'] = self.content.data.get_custom(chk_specials=chk_specials)
    if self.has_header():
      for header in self.headers.values():
        if header.active:
          custom['slide-header_'+str(header.number)] = header.data.get_custom(chk_specials=chk_specials)
    if self.has_footer():
      for footer in self.footers.values():
        if footer.active:
          custom['slide-footer_'+str(footer.number)] = footer.data.get_custom(chk_specials=chk_specials)
    if self.has_sidebar():
      for sidebar in self.sidebars.values():
        if sidebar.active:
          custom['slide-sidebar_'+str(sidebar.number)] = sidebar.data.get_custom(chk_specials=chk_specials)
    return custom

  def get_options(self):
    """Method for getting the available data options.

    Returns
    -------
    str
      string with option_names = values pairs (without True/False custom tag)

    >>> sld1 = Slide(defaults=True)
    >>> sld1.get_options().split('\\n')[5]
    'width = 900px'
    """
    string = ['\n\nSlide Global']
    string.append(self.data.get_options())
    if self.has_header():
      for header in self.headers.values():
        string.append(header.get_options())
    if self.has_footer():
      for footer in self.footers.values():
        string.append(footer.get_options())
    if self.has_sidebar():
      for sidebar in self.sidebars.values():
        string.append(sidebar.get_options())
    string.append(self.content.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for setting theme values. The theme skeleton is passed as string.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the theme if as_list = False
    list
      a list of one string containing the css code of the theme if as_list = True

    >>> sld1 = Slide(defaults=True)
    >>> sld1.get_css(as_list=True)[0].split('\\n')[5]
    '  width: 900px;'
    """
    css = ["\n.slide {\n  display: block;\n  padding: 0;\n  margin: 0;"+super(Slide,self).get_css(only_custom=only_custom)+"\n}\n"]
    if self.has_header():
      for header in self.headers.values():
        css.append(header.get_css(only_custom=only_custom))
    if self.has_footer():
      for footer in self.footers.values():
        css.append(footer.get_css(only_custom=only_custom))
    if self.has_sidebar():
      for sidebar in self.sidebars.values():
        css.append(sidebar.get_css(only_custom=only_custom))
    css.append(self.content.get_css(only_custom=only_custom))
    if as_list:
      return css
    else:
      return ''.join(css)

  def strip(self,source):
    """Method for striping theme slide raw data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without the theme data
    """
    strip_source = self.data.strip(source)
    strip_source = self.content.strip(strip_source)
    if self.has_header():
      for hdr in self.headers.values():
        strip_source = hdr.strip(strip_source)
    if self.has_footer():
      for ftr in self.footers.values():
        strip_source = ftr.strip(strip_source)
    if self.has_sidebar():
      for sdr in self.sidebars.values():
        strip_source = sdr.strip(strip_source)
    return strip_source



#!/usr/bin/env python
"""
sidebar.py, module definition of Sidebar class.
This defines the theme of the slide sidebar element.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from ..theme_element import ThemeElement
# class definition
class Sidebar(ThemeElement):
  """
  Object for handling the presentation theme slide sidebars, their attributes and methods.
  """
  def __init__(self,number,source=None):
    """
    Parameters
    ----------
    number : int
      number of header in the global numeration
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      number of header in the global numeration
    position : {'R','L'}
      sidebar position: rigth of left respect with the slide content element
    """
    self.number = number
    self.position = None
    super(Sidebar,self).__init__(data_tag=r'theme_slide_sidebar_'+str(self.number),special_keys=['position'],class_name='slide-sidebar_'+str(self.number))
    self.data.data['display'    ] = ['block',                       False]
    self.data.data['width'      ] = ['0%',                          False]
    self.data.data['height'     ] = ['100%',                        False]
    self.data.data['font-size'  ] = ['100%',                        False]
    self.data.data['font-family'] = ['Open Sans, Arial, sans-serif',False]
    self.data.data['position'   ] = [self.position,                 False]
    self.data.data['overflow'   ] = ['hidden',                      False]
    if source:
      self.get(source)
      self.check_specials()
    return

  def check_specials(self):
    """Method for checking specials data entries.

    This theme element has the following special entries:
    1. position

    >>> source = '---theme_slide_sidebar_1 width = 15% ---endtheme_slide_sidebar_1'
    >>> sidb1 = Sidebar(number=1,source=source)
    >>> sidb1.check_specials()
    >>> sidb1.data.data['position'][0]
    'R'
    """
    super(Sidebar,self).check_specials()
    for key,val in self.data.data.items():
      if val[1]:
        if key == 'position':
          self.position = val[0]
          self.data.data[key] = [self.position,True]
    if not self.position:
      # user does not set the position, default is right
      self.position = 'R'
      self.data.data['position'] = [self.position,False]
    return

  def set_from(self,other):
    """Method for setting theme using data of other (theme element).

    Parameters
    ----------
    other : Sidebar object

    >>> source = '---theme_slide_sidebar_1 background = red ---endtheme_slide_sidebar_1'
    >>> sidb1 = Sidebar(number=1,source=source)
    >>> source = '---theme_slide_sidebar_2 width = 15% \\n position = L ---endtheme_slide_sidebar_2'
    >>> sidb2 = Sidebar(number=2,source=source)
    >>> sidb1.set_from(sidb2)
    >>> sidb1.data.data['width'][0]
    '15%'
    >>> sidb1.data.data['position'][0]
    'L'
    """
    overposition = None
    if self.data.data['position'][1]:
      overposition = self.data.data['position']
    super(Sidebar,self).set_from(other=other)
    if overposition:
      self.data.data['position'] = overposition
    self.check_specials()
    return

  def adjust_dims(self,headers,footers):
    """Method for adjusting sidebar dimensions accordingly to the settings of other elements of the slide theme.

    Parameters
    ----------
    headers : dict
      dictionary with values of Header objects, being the list of headers defined into the slide
    footers : dict
      dictionary with values of Footer objects, being the list of footers defined into the slide

    >>> from .header import Header
    >>> source = '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> myheader = Header(number=1,source=source)
    >>> source = '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> mysidebar = Sidebar(number=1,source=source)
    >>> mysidebar.adjust_dims(headers={'1':myheader},footers={})
    >>> mysidebar.data.data['height'][0]
    '90%'
    """
    s_h = 100 # slide sidebar height (in percent %); should be always 100% initially
    if len(headers)>0:
      for header in headers.values():
        if header.active:
          s_h -= int(header.data.data['height'][0].strip('%'))
    if len(footers)>0:
      for footer in footers.values():
        if footer.active:
          s_h -= int(footer.data.data['height'][0].strip('%'))
    self.data.data['height'] = [str(s_h)+'%',True]
    return

  def get_options(self):
    """Method for getting the available data options.

    >>> source = '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> mysidebar = Sidebar(number=1,source=source)
    >>> mysidebar.get_options()
    '\\n\\nSlide Sidebar\\nmetadata = []\\nactive = True\\ndisplay = block\\nwidth = 10%\\nheight = 100%\\nfont-size = 100%\\nfont-family = Open Sans, Arial, sans-serif\\nposition = R\\noverflow = hidden'
    """
    string = ['\n\nSlide Sidebar']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.

    Parameters
    ----------
    only_custom : bool, optional
      consider only the (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the element if as_list = False
    list
      a list of one string containing the css code of the element if as_list = True

    >>> source = '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> mysidebar = Sidebar(number=1,source=source)
    >>> mysidebar.get_css(only_custom=True)
    '\\n .slide-sidebar_1 {\\n  float: left;\\n  width: 10%;\\n}\\n'
    """
    css = "\n .slide-sidebar_"+str(self.number)+" {\n  float: left;"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css



#!/usr/bin/env python
"""
slide.py, module definition of Slide class.
This defines a slide of the presentation.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
from collections import OrderedDict
import re
# modules not in the standard library
from yattag import Doc
# MaTiSSe.py modules
from ..config import __config__
from ..theme import box
from ..theme import columns
from ..theme import figure
from ..theme import note
from ..theme import table
from ..theme.theme import Theme
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
from ..utils.source_editor import illuminate_protected as illuminate


# class definition
class Slide(object):
  """
  Slide is an object that handles a single slide, its attributes and methods.

  Attributes
  ----------
  slides_number : int
    global number of slides (equals to the number of Slide instances)
  """
  slides_number = 0

  @classmethod
  def reset(cls):
    """Method resetting Slide to initial values."""
    cls.slides_number = 0
    return

  def __init__(self, raw_body='', title='', data=None, theme=None, local_number=1):
    """
    Parameters
    ----------
    raw_body : str, optional
      string containing the body of the slide in raw format
    title : str, optional
      slide title
    data : OrderedDict object, optional
      slide metadata
    local_number : int, optional
      subsection number in local-to-section numeration

    Attributes
    ----------
    raw_body : str
      string containing the body of the slide in raw format
    number : int
      slide number in global numeration
    local_number : int
      subsection number in local-to-subsection numeration
    title : str
      slide title
    data : OrderedDict object
      slide metadata
    overtheme : Theme object
      overriding theme of the current slide
    """
    Slide.slides_number += 1
    self.raw_body = raw_body
    self.number = Slide.slides_number
    self.local_number = local_number
    self.title = title
    self.data = OrderedDict()
    if data:
      for key, val in data.items():
        self.data[key] = val
    self.data['slidetitle'] = self.title
    self.data['slidenumber'] = str(self.number)
    self.overtheme = None
    self.__get_overtheme(theme=theme)
    # metadata autoparsing...
    for meta in ['sectiontitle', 'subsectiontitle', 'slidetitle']:
      if meta in self.data:
        self.data[meta] = self.parse_metadata(source=self.data[meta])
    return

  def __get_overtheme(self, theme=None):
    """Method for checking if the slide has a personal theme overriding the global one.

    Parameters
    ----------
    theme: Theme object, optional
      base theme used to set other overtheme data not being defined
    """
    source = seditor.get_overtheme(source=self.raw_body)
    if source:
      self.overtheme = Theme(source=source)
      if theme:
        self.overtheme.set_from(other=theme)
      self.overtheme.slide.adjust_dims()
      self.raw_body = seditor.strip_overtheme(self.raw_body)
      if __config__.verbose:
        print('Found overriding theme for slide n. ' + str(self.number))
        print(self.get_css(only_custom=True))
    return

  def get_css(self, only_custom=False):
    """Method returning slide's css.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data

    Returns
    -------
    str
      a string containing the css code of the theme
    """
    css = []
    if self.overtheme:
      css = self.overtheme.slide.get_css(only_custom=only_custom, as_list=True)
    if len(css) > 0:
      return ''.join(['\n#slide-' + str(self.number) + c[1:] for c in css])
    else:
      return ''

  def put_attributes(self, doc):
    """Method for putting html attibutes of the slide.

    Parameters
    ----------
    """
    doc.attr(('id', 'slide-' + str(self.number)))
    doc.attr(('title', self.title))
    if 'sectiontitle' in self.data:
      doc.attr(('sectiontitle', self.data['sectiontitle']))
    if 'sectionnumber' in self.data:
      doc.attr(('sectionnumber', self.data['sectionnumber']))
    if 'subsectiontitle' in self.data:
      doc.attr(('subsectiontitle', self.data['subsectiontitle']))
    if 'subsectionnumber' in self.data:
      doc.attr(('subsectionnumber', self.data['subsectionnumber']))
    return

  def metadata_to_html(self, metadata, style=None):
    """Method for converting slide level metadata to html.

    Parameters
    ----------
    metadata : str
      metadata key
    style : str, optional
      css style of metadata tag

    Returns
    -------
    str
      html string containing the metadata
    """
    doc = Doc()
    with doc.tag('span', klass='metadata'):
      if style:
        doc.attr(style=style)
      doc.asis(self.data[metadata])
    return doc.getvalue()

  def parse_metadata(self, source):
    """Method for parsing metadata of slide level that are not parsed from presentation metadata parsing.

    The slide level metadata are:
    + sectiontitle: the title of each section that is obtained parsing your source;
    + sectionnumber: the number of each section that is obtained parsing your source;
    + subsectiontitle: the title of each subsection that is obtained parsing your source;
    + subsectionnumber: the number of each subsection that is obtained parsing your source;
    + slidetitle: the title of each slide that is obtained parsing your source;
    + slidenumber: the number of each slide that is obtained parsing your source.
    """
    protected, obfuscate_source = obfuscate(source=source)
    for meta in ['sectiontitle', 'sectionnumber', 'subsectiontitle', 'subsectionnumber', 'slidetitle', 'slidenumber']:
      if meta != 'toc':
        regex = re.compile(r"\$" + meta + r"(\[(?P<style>.*?)\])*", re.DOTALL)
        for match in re.finditer(regex, obfuscate_source):
          style = None
          if match.group('style'):
            style = str(match.group('style'))
          obfuscate_source = re.sub(regex, self.metadata_to_html(metadata=meta, style=style), obfuscate_source, 1)
    return illuminate(source=obfuscate_source, protected_contents=protected)

  def raw_body_parse(self):
    """Method for parsing raw_body.

    Returns
    -------
    str
      string containing the parsed raw_body
    """
    tokens = columns.tokenize(source=self.raw_body)
    parsed_body = ''
    for token in tokens:
      if token[0] == 'unknown':
        content = box.parse(token[1])
        content = figure.parse(content)
        content = table.parse(content)
        content = note.parse(content)
        parsed_body += '\n' + seditor.md_convert(content)
      elif token[0] == 'columns':
        parsed_body += '\n' + columns.parse(source=token[1])
    return self.parse_metadata(source=parsed_body)

  def to_html(self, position, theme):
    """Method for converting slide content into html format.

    Parameters
    ----------
    position : SlidePosition object
      current slide position
    doc : yattag.Doc object
      the main html doc
    theme : Theme object
      the base theme
    """
    doc = Doc()
    with doc.tag('div'):
      self.put_attributes(doc=doc)
      # get slide positioning data
      actual_theme = None
      if self.overtheme:
        actual_theme = self.overtheme.slide
      else:
        actual_theme = theme
      position.set_position(theme=actual_theme)
      if self.title != '$overview':
        doc.attr(('class', 'step slide'))
        doc.attr(('data-x', str(position.position[0])))
        doc.attr(('data-y', str(position.position[1])))
        doc.attr(('data-z', str(position.position[2])))
        doc.attr(('data-scale', str(position.scale)))
        doc.attr(('data-rotate-x', str(position.rotation[0])))
        doc.attr(('data-rotate-y', str(position.rotation[1])))
        doc.attr(('data-rotate-z', str(position.rotation[2])))
        # inserting elements
        for header in actual_theme.loop_over_headers():
          header.to_html(doc=doc, metadata=self.data)
        for sidebar in actual_theme.loop_over_sidebars():
          if sidebar.position == 'L':
            sidebar.to_html(doc=doc, metadata=self.data)
        actual_theme.content.to_html(doc=doc, content='\n' + self.raw_body_parse())
        for sidebar in actual_theme.loop_over_sidebars():
          if sidebar.position == 'R':
            sidebar.to_html(doc=doc, metadata=self.data)
        for footer in actual_theme.loop_over_footers():
          footer.to_html(doc=doc, metadata=self.data)
      else:
        doc.attr(('class', 'step overview'))
        doc.attr(('style', ''))
        doc.attr(('data-x', str(position.position[0])))
        doc.attr(('data-y', str(position.position[1])))
        doc.attr(('data-z', str(position.position[2])))
        # doc.attr(('data-scale',str(position.scale)))
        doc.attr(('data-scale', '100'))
    return doc.getvalue()



#!/usr/bin/env python
"""
presentation.py, module definition of Presentation class.
This defines the Presentation object.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import os
import re
from shutil import copytree, rmtree
# modules not in the standard library
from yattag import Doc, indent
# MaTiSSe.py modules
from ..config import __config__
from ..theme.slide.position import Position
from ..theme.theme import Theme
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
from ..utils.source_editor import illuminate_protected as illuminate
from .metadata import Metadata
from .regexs import __regex_section__
from .section import Section
from .slide import Slide
from .subsection import Subsection
from .titlepage import Titlepage
from .toc import TOC


# class definition
class Presentation(object):
  """
  Object for handling the presentation.
  """
  def __init__(self, source=None, defaults=False, reset_theme=False):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source
    defaults : bool, optional
      flag for activatin the creation of a presentation istance
      having one of each element available with the default
      settings
    reset_theme : bool, optional
      reset theme components

    Attributes
    ----------
    metadata : Metadata object
      presentation metadata
    theme : Theme object
    sections : list
      list of sections composing the presentation
    toc : TOC object
      presentation Table of Contents
    pos : Position object
      position of the current slide
    svgpath_pos : list
      list of Position objects, one for each slide (computing from a svg file)
    center : Position object
      position of the center slide (overview slides)
    """
    self.__reset()
    self.metadata = Metadata()
    self.theme = Theme(defaults=defaults, reset=reset_theme)
    self.sections = None
    self.remainder = None
    self.titlepage = Titlepage()
    self.toc = TOC()
    self.pos = Position()
    self.svgpath_pos = None
    self.center = None
    if source:
      self.__get(source)
    return

  def __str__(self):
    string = 'Presentation preamble\n'
    string += str(self.metadata)
    string += str(self.theme)
    return string

  def __get_slides_position_fromsvg(self, svgfile, slides_number=1, slide_width=900, slide_height=700, slide_offset=1):
    from svg.path import parse_path
    from lxml import etree
    # computing minimum and maximum coordinates defined into the svg file
    paths = []
    tree = etree.parse(open(svgfile, 'r'))
    for element in tree.iter():
      if element.tag.split("}")[1] == 'path':
        path = parse_path(element.get("d"))
        for line in path:
          paths.append(line)
    minimum = [float("inf"), float("inf")]
    maximum = [-float("inf"), -float("inf")]
    for line in paths:
      minimum = [min(minimum[0], line.start.real, line.end.real), min(minimum[1], line.start.imag, line.end.imag)]
      maximum = [max(maximum[0], line.start.real, line.end.real), max(maximum[1], line.start.imag, line.end.imag)]
    # computing canvas scaling factors
    canvas_scale = [1., 1.]
    canvas_scale[0] = (slide_width * (1. + slide_offset / 100) * slides_number) / (maximum[0] - minimum[0]) / 4.
    canvas_scale[1] = (slide_height * (1. + slide_offset / 100) * slides_number) / (maximum[1] - minimum[1]) / 4.
    # computing slides positions
    self.svgpath_pos = []
    slides_on_line = int(slides_number / len(paths))
    for line in paths[:-1]:
      for i in range(slides_on_line):
        self.svgpath_pos.append(Position(pos=[line.point(0. + float(i) / float(slides_on_line + 1)).real * canvas_scale[0], line.point(0. + float(i) / float(slides_on_line + 1)).imag * canvas_scale[1], 0]))
    if slides_number % slides_on_line != 0:
      for i in range(slides_on_line + (slides_number % slides_on_line)):
        self.svgpath_pos.append(Position(pos=[path[-1].point(0. + float(i) / float(slides_on_line + 1)).real * canvas_scale[0], path[-1].point(0. + float(i) / float(slides_on_line + 1)).imag * canvas_scale[1], 0]))
    else:
      for i in range(slides_on_line):
        self.svgpath_pos.append(Position(pos=[path[-1].point(0. + float(i) / float(slides_on_line + 1)).real * canvas_scale[0], path[-1].point(0. + float(i) / float(slides_on_line + 1)).imag * canvas_scale[1], 0]))
    # computing center for overview slides
    center = [0, 0, 0]
    center[0] = (maximum[0] + minimum[0]) * canvas_scale[0] / 2.
    center[1] = (maximum[1] + minimum[1]) * canvas_scale[1] / 2.
    return center

  def __get_theme(self, source):
    """Method for getting the presentation theme.

    Return the source without the theme.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without theme data
    """
    self.theme.get(source)
    return self.theme.strip(source)

  def __get_sections(self, source):
    """Method for getting the sections contained into the source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    sections = []
    self.sections = []
    purged_source = seditor.purge_codes(source)
    for match in re.finditer(__regex_section__, purged_source):
      sections.append([match.group('expr'), match.start(), match.end()])
    if len(sections) == 0:
      # there is no section thus crate one with no title as a generic container
      self.sections.append(Section(raw_body=source, data=self.metadata.data.data))
    else:
      for scs, section in enumerate(sections):
        if scs == 0:
          if section[1] != 0:
            self.get_remainder(source, section[1])
        if scs < len(sections) - 1:
          raw_body = source[section[2] + 1:sections[scs + 1][1]]
        else:
          raw_body = source[section[2] + 1:]
        self.sections.append(Section(raw_body=raw_body, title=section[0], data=self.metadata.data.data))
    return

  def __get(self, source):
    """Method for getting the presentation from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    protected, obfuscate_source = obfuscate(source=source)
    strip_source = self.metadata.get(source=obfuscate_source)
    strip_source = self.__get_theme(source=strip_source)
    strip_source = self.titlepage.get(source=strip_source)
    strip_source = illuminate(source=strip_source, protected_contents=protected)
    self.__get_sections(source=strip_source)
    for section in self.sections:
      section.get_subsections()
      for subsection in section.subsections:
        subsection.get_slides(theme=self.theme)
    self.metadata.data.data['total_slides_number'] = [Slide.slides_number, True]
    for section in self.sections:
      for subsection in section.subsections:
        for slide in subsection.slides:
          slide.data['total_slides_number'] = Slide.slides_number
    self.toc.get(sections=self.sections)
    if self.theme.slide.data.data['slide-transition'][0].lower() == 'svgpath':
      self.center = Position(pos=self.__get_slides_position_fromsvg(svgfile='test.svg', slides_number=int(self.metadata.data.data['total_slides_number'][0]), slide_offset=200))
    else:
      self.center = Position()
    if __config__.verbose:
      print('\nTable of Contents')
      print(self.toc)
    return

  @staticmethod
  def __reset():
    """Method resetting presentation to initial values."""
    Section.reset()
    Slide.reset()
    Subsection.reset()
    return

  def get_remainder(self, source, end_remainder):
    """Method for getting the remainder of the source in case there are data subsections/slides
    before the first section defined, i.e. bad use of sections.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    end_remainder : int
      last character of remainder string into the raw_body
    """
    self.remainder = source[0:end_remainder]
    if __config__.verbose:
      message = ['\nAttention: found a bad usage of "# section" presentation sectioning!']
      message.append('\nThe data:\n"""\n')
      message.append(self.remainder)
      message.append('"""\nis placed before the first section defined')
      message.append('\nThe correct usage is the follwong:')
      message.append('\n1. place the subsection/data slides (e.g. "## subsection ### sllide") after the first defined section;')
      message.append('\n2. not use at all the section partitioning.\n')
      print(''.join(message))
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['Presentation metadata']
    string.append(self.metadata.get_options())
    string.append(self.theme.get_options())
    return ''.join(string)

  def get_css(self):
    """Create the css theme.

    Returns
    -------
    str
      string containing the css theme.
    """
    css = self.theme.get_css(only_custom=False)
    if self.titlepage.found:
      css += self.titlepage.get_css(only_custom=True)
    if self.sections:
      for section in self.sections:
        if section.subsections:
          for subsection in section.subsections:
            if subsection.slides:
              for slide in subsection.slides:
                css += slide.get_css(only_custom=True)
    return css

  def to_html(self):
    """Method for producing and html string document from presentation object."""
    doc, tag, text = Doc().tagtext()
    doc.asis('<!DOCTYPE html>')
    with tag('html'):
      doc.attr(title=self.metadata.data.data['title'][0])
      with tag('head'):
        doc.stag('meta', charset='utf-8')
        doc.stag('meta', author=' and '.join(self.metadata.data.data['authors'][0]))
        with tag('title'):
          text(self.metadata.data.data['title'][0])
        doc.stag('meta', subtitle=self.metadata.data.data['subtitle'][0])
        doc.stag('link', rel='stylesheet', href='css/normalize.css')
        if __config__.highlight:
          doc.stag('link', rel='stylesheet', href='js/highlight/styles/' + __config__.highlight_style)
        doc.stag('link', rel='stylesheet', href='css/theme.css')
      with tag('body', onload="resetCountdown(" + self.metadata.data.data['max_time'][0] + ");"):
        with tag('div', id='impress'):
          if self.titlepage.found:
            if self.theme.slide.data.data['slide-transition'][0].lower() == 'svgpath':
              pos = self.svgpath_pos[0]
            else:
              pos = self.pos
            html = self.titlepage.to_html(position=pos, theme=self.theme.slide)
            html = self.metadata.parse(html)
            html = self.toc.parse(html)
            doc.asis(html)
          for section in self.sections:
            for subsection in section.subsections:
              for slide in subsection.slides:
                if slide.title == '$overview':
                  pos = self.center
                elif self.theme.slide.data.data['slide-transition'][0].lower() == 'svgpath':
                  pos = self.svgpath_pos[slide.number - 1]
                else:
                  pos = self.pos
                html = slide.to_html(position=pos, theme=self.theme.slide)
                html = self.metadata.parse(html)
                html = self.toc.parse(html, current=[int(slide.data['sectionnumber']), int(slide.data['subsectionnumber']), slide.number])
                doc.asis(html)
        with tag('script'):
          doc.attr(src='js/countDown.js')
        with tag('script'):
          doc.attr(src='js/impress.js')
        with tag('script'):
          doc.asis('impress().init();')
        if __config__.online_mathjax:
          with tag('script'):
            doc.attr(('type', 'text/javascript'))
            doc.attr(src='http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML')
        else:
          with tag('script'):
            doc.attr(('type', 'text/x-mathjax-config'))
            doc.text("""
              MathJax.Hub.Config({
                extensions: ["tex2jax.js"],
                jax: ["input/TeX", "output/HTML-CSS"],
                tex2jax: {
                  inlineMath: [ ['$','$'], ["\\(","\\)"] ],
                  displayMath: [ ['$$','$$'], ["\\[","\\]"] ],
                  processEscapes: true
                },
                "HTML-CSS": { availableFonts: ["Neo-Euler"] }
              });
            """)
          with tag('script'):
            doc.attr(('type', 'text/javascript'))
            doc.attr(src='js/MathJax/MathJax.js')
        if __config__.highlight:
          with tag('script'):
            doc.attr(src='js/highlight/highlight.pack.js')
          with tag('script'):
            doc.text("""hljs.initHighlightingOnLoad();""")
    if __config__.indented:
      return indent(doc.getvalue())
    else:
      return doc.getvalue()

  def save(self, output):
    """Method for saving the html form of presentation into external file.

    Parameters
    ----------
    output : str
      output path
    """
    with open(output + 'index.html', 'w') as html:
      html.write(self.to_html())
    # copy user defined directories if set
    if len(self.metadata.data.data['dirs_to_copy'][0]) > 0:
      for data in self.metadata.data.data['dirs_to_copy'][0]:
        if os.path.exists(output + data):
          rmtree(output + data)
        copytree(data, output + data)
    # writing css theme
    with open(output + 'css/theme.css', 'w') as css_theme:
      css_theme.writelines(self.get_css())
    return



#!/usr/bin/env python
"""
toc.py, module definition of TOC class.
This defines the Table of Contents of the presentation.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# modules not in the standard library
from yattag import Doc
# global variables
# regular expressions
__retoc__ = re.compile(r"\$toc(\((?P<deep>[1-3])\))*",re.DOTALL)
# class definition
class TOC(object):
  """
  Object handling the table of contents of presntation.
  """
  def __init__(self,deep=3):
    """
    Parameters
    ----------
    deep : {1,2,3}, optional
      depth of printed TOC; 1 => print only sections, 2 => print sections and subsections, 3=> print sections, subsections and slides

    Attributes
    ----------
    sections : list
      list of sections with the following form
      sections = [ #section_number, 'section title', [list of subsections], #slide_number_at_which_section_starts ];
      the subsections list has the following form
      subsections = [ #subsection_number, 'subsection title', [list of slides], #local_subsection_number, #slide_number_at_which_subsection_starts ];
      the slides list has the following form
      subsections = [ #slide_number, 'slide title', #local_slide_number ];
    deep : {1,2,3}
      depth of printed TOC; 1 => print only sections, 2 => print sections and subsections, 3=> print sections, subsections and slides
    """
    self.sections = []
    self.deep = deep
    return

  def __str__(self):
    return self.pstr()

  @staticmethod
  def print_section(section,html=False,current=None):
    """Method for printing section data.

    Parameters
    ----------
    section: list
      section data
    html: bool, optional
      activate html tags
    current: list, optional
      list containing current section, subsection and slide number used
      for highlighting current slide into the TOC

    Returns
    -------
    str
      string containing the pretty printed section data
    """
    if html:
      doc, tag, text = Doc().tagtext()
      with tag('a',href='#slide-'+str(section[3])):
        if current and current[0] == section[0]:
          with tag('span',klass='toc-section emph'):
            text(str(section[0])+' '+section[1])
        else:
          with tag('span',klass='toc-section'):
            text(str(section[0])+' '+section[1])
      string = '\n'+doc.getvalue()
    else:
      string = '\n'+str(section[0])+' '+section[1]
    return string

  @staticmethod
  def print_subsection(section,subsection,html=False,current=None):
    """Method for printing subsection data.

    Parameters
    ----------
    section: list
      section data
    subsection: list
      subsection data
    html: bool, optional
      activate html tags
    current: list, optional
      list containing current section, subsection and slide number used
      for highlighting current slide into the TOC

    Returns
    -------
    str
      string containing the pretty printed subsection data
    """
    if html:
      doc, tag, text = Doc().tagtext()
      with tag('a',href='#slide-'+str(subsection[4])):
        if current and current[0] == section[0] and current[1] == subsection[0]:
          with tag('span',klass='toc-subsection emph'):
            text('  '+str(section[0])+'.'+str(subsection[3])+' '+subsection[1])
        else:
          with tag('span',klass='toc-subsection'):
            text('  '+str(section[0])+'.'+str(subsection[3])+' '+subsection[1])
      string = '\n'+doc.getvalue()
    else:
      string = '\n'+'  '+str(section[0])+'.'+str(subsection[3])+' '+subsection[1]
    return string

  @staticmethod
  def print_slide(section,subsection,slide,html=False,current=None):
    """Method for printing slideta.

    Parameters
    ----------
    section: list
      section data
    subsection: list
      subsection data
    slide: list
      slide data
    html: bool, optional
      activate html tags
    current: list, optional
      list containing current section, subsection and slide number used
      for highlighting current slide into the TOC

    Returns
    -------
    str
      string containing the pretty printed slide data
    """
    if html:
      doc, tag, text = Doc().tagtext()
      with tag('a',href='#slide-'+str(slide[0])):
        if current and current[0] == section[0] and current[1] == subsection[0] and current[2] == slide[0]:
          with tag('span',klass='toc-slide emph'):
            text('    '+str(section[0])+'.'+str(subsection[3])+'.'+str(slide[2])+' '+slide[1])
        else:
          with tag('span',klass='toc-slide'):
            text('    '+str(section[0])+'.'+str(subsection[3])+'.'+str(slide[2])+' '+slide[1])
      string = '\n'+doc.getvalue()
    else:
      string = '\n'+'    '+str(section[0])+'.'+str(subsection[3])+'.'+str(slide[2])+' '+slide[1]
    return string

  def pstr(self,html=False,current=None,deep=None):
    """Method powering __str__ obtaining a customizable pretty printer.

    Parameters
    ----------
    html: bool, optional
      activate html tags
    current: list, optional
      list containing current section, subsection and slide number used
      for highlighting current slide into the TOC
    deep : {1,2,3}, optional
      depth of printed TOC; 1 => print only sections, 2 => print sections and subsections, 3=> print sections, subsections and slides
    """
    deep = deep
    if not deep:
      deep = self.deep
    string = ''
    for section in self.sections:
      string += self.print_section(section=section,html=html,current=current)
      if deep > 1:
        for subsection in section[2]:
          string += self.print_subsection(section=section,subsection=subsection,html=html,current=current)
          if deep > 2:
            for slide in subsection[2]:
              string += self.print_slide(section=section,subsection=subsection,slide=slide,html=html,current=current)
    return string

  def get(self,sections):
    """
    Method for building TOC from presentation sections.
    """
    for sec,section in enumerate(sections):
      self.sections.append([ int(section.number),section.title,[],0 ])
      for sub,subsection in enumerate(section.subsections):
        self.sections[sec][2].append([ int(subsection.number),subsection.title,[],int(subsection.local_number),0 ])
        for slide in subsection.slides:
          self.sections[sec][2][sub][2].append([ int(slide.number),slide.title,int(slide.local_number) ])
          if slide.data['first_of_sec']:
            self.sections[sec][3] = slide.number
          if slide.data['first_of_subsec']:
            self.sections[sec][2][sub][4] = slide.number
    return

  def parse(self,source,current=None):
    """Method for substituting $toc with its pretty printed version.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    current: list, optional
      list containing current section, and subsection number used
      for highlighting current slide into the TOC
    """
    parsed_source = source
    for match in re.finditer(__retoc__,parsed_source):
      deep = match.group('deep')
      if deep:
        deep = int(deep)
      doc = Doc()
      with doc.tag('div',klass='toc'):
        doc.asis(self.pstr(html=True,current=current,deep=deep))
      parsed_source = re.sub(__retoc__,doc.getvalue(),parsed_source,1)
      doc = None
    return parsed_source



#!/usr/bin/env python
"""
source_editor.py, module definition of SourceEditor class.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import os
import re
import sys
# modules not in the standard library
import markdown
from .mdx_mathjax import MathJaxExtension
try:
  from markdown_checklist.extension import ChecklistExtension
  __mdx_checklist__ = True
except ImportError:
  __mdx_checklist__ = False
# global variables
__regex_codeblock__ = re.compile(r"(?P<cblock>[`]{3}.*?[`]{3})", re.DOTALL)
__regex_codeblock_html__ = re.compile(r"(?P<cblock>\<code.*?\<\/code\>)", re.DOTALL)
__regex_codeinline__ = re.compile(r"(?P<cline>[`]{1}.*?[`]{1})", re.DOTALL)
__regex_overtheme__ = re.compile(r"(?P<ostheme>[-]{3}slide(?P<block>.*?)[-]{3}endslide)", re.DOTALL)
__regex_include__ = re.compile(r"\$include\((?P<include>.*?)\)")
__regex_protected__ = re.compile(r"(\$PROTECTED-(?P<num>[0-9]*))")


# class definition
class SourceEditor(object):
  """
  Object for editing source stream:

  - get from source necessary data;
  - purge from source uncessary data;
  - strip out from source uncessary data.
  """
  def __init__(self):
    """
    Attributes
    ----------
    regex_codeblock : re.compile object
    regex_codeinline : re.compile object
    regex_overtheme : re.compile object
    mkd : markdown.Markdown object
      markdown converter
    """
    self.regex_codeblock = __regex_codeblock__
    self.regex_codeinline = __regex_codeinline__
    self.regex_overtheme = __regex_overtheme__
    if __mdx_checklist__:
      self.mkd = markdown.Markdown(output_format='html5',
                                   extensions=['smarty',
                                               'extra',
                                               ChecklistExtension(),
                                               MathJaxExtension()])
    else:
      self.mkd = markdown.Markdown(output_format='html5',
                                   extensions=['smarty',
                                               'extra',
                                               MathJaxExtension()])
    return

  def md_convert(self, source, no_p=False):
    """Convert markdown source to html.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    no_p : bool, optional
      if True the converted contents is not inserted into the <p></p> tags

    Returns
    -------
    str
      converted source
    """
    markup = self.mkd.reset().convert(source)
    if no_p:
      p_start = '<p>'
      p_end = '</p>'
      if markup.startswith(p_start) and markup.endswith(p_end):
        markup = markup[len(p_start):-len(p_end)]
    return markup

  @staticmethod
  def get(regex, source, group_name=None):
    """Method for getting blocks of characters matching 'regex' from a source string.

    Parameters
    ----------
    regex : re.compile object
      regular expression to match
    source : str
      string (as single stream) containing the source
    group_name : str, optional
      name of the group to get; if no name is passed the whole 'regex' is gotten

    Returns
    -------
    str
      (first) matching block of source; if nothing matches None is returned; if group
      name is not passed the whole match is returned

    >>> seditor = SourceEditor()
    >>> source = '---test contents ---endtest other contents'
    >>> regex = re.compile(r"(?P<test>[-]{3}test.*?[-]{3}endtest)",re.DOTALL)
    >>> seditor.get(regex,source,'test')
    '---test contents ---endtest'
    >>> seditor.get(regex,source,'testNone')
    '---test contents ---endtest'
    >>> regex = re.compile(r"(?P<test>[-]{3}wrongtest.*?[-]{3}endwrongtest)",re.DOTALL)
    >>> seditor.get(regex,source,'test')
    """
    match = re.match(regex, source)
    if match:
      if group_name and group_name in match.groupdict():
        return match.group(group_name)
      else:
        return match.group()
    return None

  def get_codeblocks(self, source):
    """Method for getting code blocks from a source string.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      (first) matching code blocks of source; if nothing matches None is returned

    >>> seditor = SourceEditor()
    >>> source = '``` my code block ``` other contents'
    >>> seditor.get_codeblocks(source)
    '``` my code block ```'
    """
    return self.get(regex=__regex_codeblock__, group_name='cblock', source=source)

  def get_codeinlines(self, source):
    """Method for getting code inlines from a source string.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      (first) matching code blocks of source; if nothing matches None is returned

    >>> seditor = SourceEditor()
    >>> source = '` my code inline ` other contents'
    >>> seditor.get_codeinlines(source)
    '` my code inline `'
    """
    return self.get(regex=__regex_codeinline__, group_name='cline', source=source)

  def get_overtheme(self, source):
    """Method for getting blocks of characters defining the overriding slide theme from a source string.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      (first) matching overriding theme block of source; if nothing matches None is returned
    """
    return self.get(regex=__regex_overtheme__, group_name='block', source=source)

  @staticmethod
  def purge(regex, source, group_name=None, strip=False):
    """Method for removing blocks of characters matching 'regex' from a source string
    and replacing with an equivalent number of spaces as the removed characters.

    Parameters
    ----------
    regex : re.compile object
      regular expression to match
    source : str
      string (as single stream) containing the source
    group_name : str, optional
      name of the group to purge; if no name is passed the whole 'regex' is purged
    strip : bool, optional
      if true the matching characters is simply stripped out and replacing with an equivalent
      number of spaces

    Returns
    -------
    str
      purged string

    >>> seditor = SourceEditor()
    >>> source = '---test contents ---endtest other contents'
    >>> regex = re.compile(r"(?P<test>[-]{3}test.*?[-]{3}endtest)",re.DOTALL)
    >>> seditor.purge(regex,source,'test')
    '                            other contents'
    """
    purged_source = source
    for match in re.finditer(regex, purged_source):
      if strip:
        sub = ''
      else:
        if group_name:
          sub = ' ' * len(match.group(group_name))
        else:
          sub = ' ' * len(match.group())
      purged_source = re.sub(regex, sub, purged_source, 1)
    return purged_source

  def purge_codeblocks(self, source):
    """Method for removing code blocks from a source string and
    replacing with an equivalent number of spaces as the removed characters.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      purged string

    >>> seditor = SourceEditor()
    >>> source = '``` my code block ``` other contents'
    >>> seditor.purge_codeblocks(source)
    '                      other contents'
    """
    return self.purge(regex=__regex_codeblock__, group_name='cblock', source=source)

  def purge_codeinlines(self, source):
    """Method for removing code inlines from a source string and
    replacing with an equivalent number of spaces as the removed characters.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      purged string

    >>> seditor = SourceEditor()
    >>> source = '` my code inline ` other contents'
    >>> seditor.purge_codeinlines(source)
    '                   other contents'
    """
    return self.purge(regex=__regex_codeinline__, group_name='cline', source=source)

  def purge_codes(self, source):
    """Method for removing code (blocks and inline) from a source string and
    replacing with an equivalent number of spaces as the removed characters.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      purged string

    >>> seditor = SourceEditor()
    >>> source = '``` my code block ``` contents ` my code inline ` other contents'
    >>> seditor.purge_codes(source)
    '                      contents                    other contents'
    """
    source = self.purge_codeinlines(source=source)
    return self.purge_codeblocks(source=source)

  def purge_overtheme(self, source):
    """Method for removing overriding slide themes blocks from a source string and
    replacing with an equivalent number of spaces as the removed characters.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      purged string

    >>> seditor = SourceEditor()
    >>> source = '---slide contents ---endslide other contents'
    >>> seditor.purge_overtheme(source)
    '                              other contents'
    """
    return self.purge(regex=__regex_overtheme__, group_name='ostheme', source=source)

  def strip(self, regex, source, group_name=None):
    """Method for striping out blocks of characters matching 'regex' from a source string.

    Parameters
    ----------
    regex : re.compile object
      regular expression to match
    source : str
      string (as single stream) containing the source
    group_name : str, optional
      name of the group to strip; if no name is passed the whole 'regex' is stripped

    Returns
    -------
    str
      stripped string

    >>> seditor = SourceEditor()
    >>> source = '---test contents ---endtest other contents'
    >>> regex = re.compile(r"(?P<test>[-]{3}test.*?[-]{3}endtest)",re.DOTALL)
    >>> seditor.strip(regex,source,'test')
    ' other contents'
    """
    return self.purge(regex=regex, source=source, group_name=group_name, strip=True)

  def strip_overtheme(self, source):
    """Method for striping out overriding slide theme blocks from a source string.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      stripped string

    >>> seditor = SourceEditor()
    >>> source = '---slide contents ---endslide other contents'
    >>> seditor.strip_overtheme(source)
    ' other contents'
    """
    protected, obfuscate_source = obfuscate_codeblocks(source=source)
    obfuscate_source = self.strip(regex=__regex_overtheme__, source=obfuscate_source)
    return illuminate_protected(source=obfuscate_source, protected_contents=protected)

  @staticmethod
  def includes(source):
    """Include other sources into source.

    It parse source for matching "include" regular expressions and for each include found
    the external sources is included "in place" into source. It is recursive!

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source with included other sources
    """
    protected, obfuscate_source = obfuscate_codeblocks(source=source)
    if __regex_include__.search(obfuscate_source):
      for match in re.finditer(__regex_include__, obfuscate_source):
        include_file = match.group('include')
        if not os.path.exists(include_file):
          sys.stderr.write('Error: cannot include "' + include_file + '"')
          sys.exit(1)
        else:
          with open(include_file, 'r') as inc:
            other_source = repr(inc.read())
          obfuscate_source = re.sub(__regex_include__, other_source, obfuscate_source, 1)
      return SourceEditor.includes(source=illuminate_protected(source=obfuscate_source, protected_contents=protected))
    return illuminate_protected(source=obfuscate_source, protected_contents=protected)


def obfuscate_protected(source, re_protected):
  """Method for obfuscating protected contents.

  It can be often useful to temporarly obfuscate some blocks of contents for performing safely some tasks
  and then re-introducing them.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source
  re_protected : re.compile object
    regular expression of protected block

  Returns
  -------
  protected_contents : list
    list of str containing the contents of protected blocks
  str
    source with protected contents obfuscated and replaced by a safe placeholder

  >>> source = '``` my code block ``` other contents'
  >>> prot, ob_source = obfuscate_protected(source,__regex_codeblock__)
  >>> prot[0][2]
  '``` my code block ```'
  >>> ob_source
  '$PROTECTED-1 other contents'
  """
  obfuscate_source = source
  protected_contents = []
  for match in re.finditer(re_protected, source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(re_protected, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  return protected_contents, obfuscate_source


def obfuscate_codeblocks(source):
  """Method for obfuscating codeblocks contents.

  It can be often useful to temporarly obfuscate codeblocks contents for performing safely some tasks
  and then re-introducing them.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  protected_contents : list
    list of str containing the contents of codeblocks
  str
    source with codeblocks contents obfuscated and replaced by a safe placeholder

  >>> source = '``` my code block ``` other contents'
  >>> prot, ob_source = obfuscate_codeblocks(source)
  >>> prot[0][2]
  '``` my code block ```'
  >>> ob_source
  '$PROTECTED-1 other contents'
  """
  obfuscate_source = source
  protected_contents = []
  for match in re.finditer(__regex_codeblock__, obfuscate_source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(__regex_codeblock__, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  for match in re.finditer(__regex_codeblock_html__, obfuscate_source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(__regex_codeblock_html__, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  return protected_contents, obfuscate_source


def obfuscate_codes(source):
  """Method for obfuscating codes contents.

  It can be often useful to temporarly obfuscate codes contents for performing safely some tasks
  and then re-introducing them.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  protected_contents : list
    list of str containing the contents of codes
  str
    source with codes contents obfuscated and replaced by a safe placeholder

  >>> source = '``` my code block ``` contents ` code inline ` other contents'
  >>> prot, ob_source = obfuscate_codes(source)
  >>> prot[0][2]
  '``` my code block ```'
  >>> prot[1][2]
  '` code inline `'
  >>> ob_source
  '$PROTECTED-1 contents $PROTECTED-2 other contents'
  """
  obfuscate_source = source
  protected_contents = []
  for match in re.finditer(__regex_codeblock__, obfuscate_source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(__regex_codeblock__, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  for match in re.finditer(__regex_codeblock_html__, obfuscate_source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(__regex_codeblock_html__, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  for match in re.finditer(__regex_codeinline__, obfuscate_source):
    protected_contents.append([match.start(), match.end(), match.group()])
    obfuscate_source = re.sub(__regex_codeinline__, '$PROTECTED-' + str(len(protected_contents)), obfuscate_source, 1)
  return protected_contents, obfuscate_source


def illuminate_protected(source, protected_contents):
  """Method for re-insterting protected contents (illuminating previously obfuscated blocks).

  It can be often useful to temporarly obfuscate some blocks of contents for performing safely some tasks
  and then re-introducing (illuminating) them.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source
  protected_contents : list
    list of str containing the contents of protected blocks

  Returns
  -------
  str
    source with protected contents illuminated (reintroduced)

  >>> source = '``` my code block ``` contents ` code inline ` other contents'
  >>> prot, ob_source = obfuscate_codes(source)
  >>> illuminate_protected(ob_source,prot)
  '``` my code block ``` contents ` code inline ` other contents'
  """
  if len(protected_contents) > 0:
    illuminate_source = source
    for match in re.finditer(__regex_protected__, illuminate_source):
      illuminate_source = re.sub(__regex_protected__, protected_contents[int(match.group('num')) - 1][2], illuminate_source, 1)
    return illuminate_source
  else:
    return source


def tokenize(source, re_part, name_part):
  """Method for tokenizing source tagging parts of the source.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source
  re_part : re.compile object
    regex matching parts to be tokenized
  name_part : str
    name of parts matched

  Returns
  -------
  list
    list of tokens whose elements are ['name',source_part]; name is name_part for parts matching re_part and
    'unknown' for anything else

  >>> source = '---test contents ---endtest other contents'
  >>> regex = re.compile(r"(?P<test>[-]{3}test.*?[-]{3}endtest)",re.DOTALL)
  >>> tokenize(source,regex,'TEST')[0][1]
  '---test contents ---endtest'
  """
  protected, obfuscate_source = obfuscate_codeblocks(source=source)
  matches = []
  for match in re.finditer(re_part, obfuscate_source):
    matches.append([match.start(), match.end(), illuminate_protected(source=match.group(), protected_contents=protected)])
  if len(matches) > 0:
    tokens = []
    for mtc, match in enumerate(matches):
      if mtc == 0:
        start = 0
      else:
        start = matches[mtc - 1][1] + 1
      if match[0] != start:
        tokens.append(['unknown', illuminate_protected(source=obfuscate_source[start:match[0]], protected_contents=protected)])
      tokens.append([name_part, match[2]])
    if matches[-1][1] < len(obfuscate_source):
      tokens.append(['unknown', illuminate_protected(source=obfuscate_source[matches[-1][1] + 1:], protected_contents=protected)])
  else:
    tokens = [['unknown', source]]
  return tokens

# global variables
__initialized__ = False
if not __initialized__:
  __source_editor__ = SourceEditor()
  __initialized__ = True



#!/usr/bin/env python
"""
section.py, module definition of Section class.
This defines a section of the presentation.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
from collections import OrderedDict
import re
# MaTiSSe.py modules
from ..config import __config__
from ..utils.source_editor import __source_editor__ as seditor
from .regexs import  __regex_subsection__
from .subsection import Subsection
# class definition
class Section(object):
  """
  Object for handling a single section, its attributes and methods.

  Attributes
  ----------
  sections_number : int
    global number of sections (equals to the number of Section instances)
  """
  sections_number = 0

  @classmethod
  def reset(cls):
    """Method resetting Section to initial values."""
    cls.sections_number = 0
    return

  def __init__(self,raw_body='',title='',data=None):
    """
    Parameters
    ----------
    raw_body : str, optional
      string containing the body of the section in raw format
    title : str, optional
      section title
    data : OrderedDict object
      section metadata

    Attributes
    ----------
    raw_body : str, optional
      slide number in global numeration
    number : int, optional
      section number in global numeration
    title : str, optional
      section title
    data : OrderedDict object
      section metadata
    subsections : list
      list of subsections
    remainder : str
      remainder of raw_body containing bad placed data slides
    """
    Section.sections_number += 1
    self.raw_body    = raw_body
    self.number      = Section.sections_number
    self.title       = title
    self.data        = OrderedDict()
    if data:
      for key,val in data.items():
        self.data[key] = val[0]
    self.data['sectiontitle' ] = self.title
    self.data['sectionnumber'] = str(self.number)
    self.subsections = None
    self.remainder = None
    return

  def get_remainder(self,end_remainder):
    """Method for getting the remainder of the source in case there are data slides
    before the first subsection defined, i.e. bad use of subsections.

    Parameters
    ----------
    end_remainder : int
      last character of remainder string into the raw_body
    """
    self.remainder = self.raw_body[0:end_remainder]
    if __config__.verbose:
      message = ['\nAttention: found a bad usage of "## subsection" presentation sectioning!']
      message.append('\nThe data:\n"""\n')
      message.append(self.remainder)
      message.append('"""\nis placed before the first subsection defined into the current section')
      message.append('\nThe correct usage is the follwong:')
      message.append('\n1. place the data slides (e.g. "### slide") after the first defined subsection;')
      message.append('\n2. not use at all the subsection partitioning.\n')
      print(''.join(message))
    return

  def get_subsections(self):
    """Method for getting the subsections contained into the section."""
    subsections = []
    self.subsections = []
    purged_source = seditor.purge_codes(self.raw_body)
    for match in re.finditer(__regex_subsection__,purged_source):
      subsections.append([match.group('expr'),match.start(),match.end()])
    if len(subsections)==0:
      # there is no subsection thus crate one with no title as a generic container
      self.subsections.append(Subsection(raw_body=self.raw_body,data=self.data))
    else:
      for ssn,subsec in enumerate(subsections):
        if ssn == 0:
          if subsec[1] != 0:
            self.get_remainder(subsec[1])
        if ssn < len(subsections)-1:
          raw_body = self.raw_body[subsec[2]+1:subsections[ssn+1][1]]
        else:
          raw_body = self.raw_body[subsec[2]+1:]
        self.subsections.append(Subsection(raw_body     = raw_body,
                                           title        = subsec[0],
                                           data         = self.data,
                                           local_number = ssn+1))
    return



#!/usr/bin/env python
"""
toc.py, module definition of TOC class.
This defines the theme of the Table of Contents.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from .theme_element import ThemeElement
# classes definition
class SectionEmph(ThemeElement):
  """
  Object for handling the section emphasized theme into TOC, its attributes and methods.
  """
  def __init__(self,source=None,skip=None):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(SectionEmph,self).__init__(data_tag=r'theme_section_emph_toc')
    self.data.data['margin'       ] = ['1% 0',             False]
    self.data.data['clear'        ] = ['both',             False]
    self.data.data['float'        ] = ['left',             False]
    self.data.data['border'       ] = ['1px solid #4788B3',False]
    self.data.data['border-radius'] = ['5px',              False]
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nTOC Section Emphasized']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\n.toc-section.emph {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css

class SubsectionEmph(ThemeElement):
  """
  Object for handling the subsection emphasized theme into TOC, its attributes and methods.
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(SubsectionEmph,self).__init__(data_tag=r'theme_subsection_emph_toc')
    self.data.data['clear'        ] = ['both',             False]
    self.data.data['float'        ] = ['left',             False]
    self.data.data['margin'       ] = ['1% 0',             False]
    self.data.data['border'       ] = ['1px solid #4788B3',False]
    self.data.data['border-radius'] = ['5px',              False]
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nTOC Subsection Emphasized']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\n.toc-subsection.emph {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css

class SlideEmph(ThemeElement):
  """
  Object for handling the slide emphasized theme into TOC, its attributes and methods.
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(SlideEmph,self).__init__(data_tag=r'theme_slide_emph_toc')
    self.data.data['margin'       ] = ['1% 0',             False]
    self.data.data['clear'        ] = ['both',             False]
    self.data.data['float'        ] = ['left',             False]
    self.data.data['border'       ] = ['1px solid #4788B3',False]
    self.data.data['border-radius'] = ['5px',              False]
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nTOC Slide Emphasized']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\n.toc-slide.emph {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css

class TOC(ThemeElement):
  """
  Object for handling the TOC theme, its attributes and methods.
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(TOC,self).__init__(data_tag=r'theme_toc')
    self.data.data['display'] = ['block',False]
    self.section_emph = SectionEmph()
    self.subsection_emph = SubsectionEmph()
    self.slide_emph = SlideEmph()
    if source:
      self.get(source)
    return

  def get(self,source):
    """Method for getting data values from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    super(TOC, self).get(source)
    self.check_specials()
    self.section_emph.get(source)
    self.subsection_emph.get(source)
    self.slide_emph.get(source)
    return

  def set_from(self,other):
    """Method for setting theme slide using data of other (theme slide element).

    Parameters
    ----------
    other : TOC object
    """
    #super(TOC, self).set_from(other=other)
    self.section_emph.set_from(other=other.section_emph)
    self.subsection_emph.set_from(other=other.subsection_emph)
    self.slide_emph.set_from(other=other.slide_emph)
    return

  def update(self,source):
    """Method for updating data from source without creating new data.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    self.get(source)
    return

  def check_specials(self):
    """Method for checking specials data entries."""
    super(TOC, self).check_specials()
    self.section_emph.check_specials()
    self.subsection_emph.check_specials()
    self.slide_emph.check_specials()
    return

  def get_custom(self,chk_specials=False):
    """Method returning only the data that have been set by users (customized) overriding default values.

    Parameters
    ----------
    chk_specials : bool, optional
      if activated handle special entries differently from standard ones

    Returns
    -------
    dict
      dictionary of customized data
    """
    custom = {}
    custom['toc'] = self.data.get_custom(chk_specials=chk_specials)
    custom['toc-section_emph'] = self.section_emph.data.get_custom(chk_specials=chk_specials)
    custom['toc-subsection_emph'] = self.subsection_emph.data.get_custom(chk_specials=chk_specials)
    custom['toc-slide_emph'] = self.slide_emph.data.get_custom(chk_specials=chk_specials)
    return custom

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nTOC']
    string.append(self.data.get_options())
    string.append(self.section_emph.get_options())
    string.append(self.subsection_emph.get_options())
    string.append(self.slide_emph.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = ["\n.toc {" + self.data.get_css(only_custom=only_custom) + "\n}\n"]
    css.append(".toc-section{\n  display: block;\n  clear: both;\n  float: left;\n  white-space: pre-wrap;\n}\n")
    css.append(".toc-subsection{\n  display: block;\n  clear: both;\n  float: left;\n  white-space: pre-wrap;\n}\n")
    css.append(".toc-slide{\n  display: block;\n  clear: both;\n  float: left;\n  white-space: pre-wrap;\n}\n")
    css.append(self.section_emph.get_css(only_custom=only_custom))
    css.append(self.subsection_emph.get_css(only_custom=only_custom))
    css.append(self.slide_emph.get_css(only_custom=only_custom))
    if as_list:
      return css
    else:
      return ''.join(css)

  def strip(self,source):
    """Method for striping toc theme raw data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without the theme data
    """
    strip_source = self.data.strip(source)
    strip_source = self.section_emph.strip(strip_source)
    strip_source = self.subsection_emph.strip(strip_source)
    strip_source = self.slide_emph.strip(strip_source)
    return strip_source



#!/usr/bin/env python
"""
regexs.py, module definition of main regular expressions.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# MaTiSSe.py modules
from ..utils.utils import __expr__
# regular expressions
__section_pre__ = r"([^#]#|^#)\s+"
__subsection_pre__ = r"([^#]##|^##)\s+"
__slide_pre__ = r"([^#]###|^###)\s+"
__regex_section__    = re.compile(__section_pre__+__expr__)
__regex_subsection__ = re.compile(__subsection_pre__+__expr__)
__regex_slide__ = re.compile(__slide_pre__+__expr__)
__regex_titlepage__ = re.compile(r"([^#]#|^#)titlepage(\[(?P<plain>plain)\])*")
__regex_endtitlepage__ = re.compile(r"(?P<sec>"+__section_pre__+r".*)|(?P<subsec>"+__subsection_pre__+r".*)|(?P<slide>"+__slide_pre__+r".*)")



#!/usr/bin/env python
"""
box.py, module definition of Box class.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# modules not in the standard library
from yattag import Doc
# MaTiSSe.py modules
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import illuminate_protected as illuminate
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
#from ..utils.source_editor import obfuscate_codes as obfuscate
from ..utils.source_editor import tokenize as generic_tokenize
# global variables
# regular expressions
__rebox__ = re.compile(r"(?P<box>\$box(?P<env>.*?)\$endbox)",re.DOTALL)
__restl__ = re.compile(r"\$style\[(?P<style>.*?)\]",re.DOTALL)
__rectn__ = re.compile(r"\$content(\((?P<ctn_type>.*?)\))*(\[(?P<ctn_options>.*?)\])*\{(?P<ctn>.*?)\}",re.DOTALL)
__recap__ = re.compile(r"\$caption(\((?P<cap_type>.*?)\))*(\[(?P<cap_options>.*?)\])*(\{(?P<cap>.*?)\})*",re.DOTALL)
# classes definition
class Box(object):
  """
  Object for handling a box. It is an environment that can contains anythings (figures, tables, notes...).

  The syntax is:

  $box
  $style[style_options]
  $caption(caption_type)[caption_options]{caption}
  $content(content_type)[content_options]{content}
  $endbox

  Note:
  1. "$style[...]" is optional; it defines the style's options for the whole box; "style_options" are valid
     css syntax statements  (e.g. "font-variant:small-caps;");
  2. "$caption(...)[...]{...}" is optional: it defines the box's caption; the "caption_type" defines the
     caption prefixing "class" (e.g. "Fig." for figures): any sentences are valid; the "caption_options" defines
     the style's options of only caption: they are valid css syntax statements (e.g.  "padding:0 2%;"); the
     "caption" (inside the {} parenthesis) defines the caption text; note that "caption_type" and "caption_options"
     are optional thus the following statements are valid:
     + $caption[font-variant:small-caps;]{My caption without caption_type};
     + $caption{My caption without caption_type and caption_options};
  3. "$content(...)[...]{...}" is not optional: it defines the box's content; the "content_type" defines the type
     of the content that can be 'figure', 'table', 'note' and 'box' for generic environments; the "content_options"
     defines the style's options of only content: they are valid css syntax statements (e.g.  "padding:0 2%;"); the
     "content" (inside the {} parenthesis) defines the content (being text, figures, tables, etc...); note that
     "content_type" and "content_options" are optional thus the following statements are valid:
     + $content[font-variant:small-caps;]{My content without content_type};
     + $content{My content without content_type and content_options};

  There are some helper (sub)classes based on Box class for handling specific environments such Figure, Table and Note.

  Note that the themes of box environments can be defined as all other theme elements in order to not have to repeat
  the styling options for each box. To this aim this module provides the "get_themes" function. The definition of such
  a theme can be stripped out by the function "strip_themes" also provided by this module.

  See Also
  --------
  Figure
  Table
  Note

  Attributes
  ----------
  boxes_number : int
    global number of boxes (equals to the number of Box instances)
  """
  boxes_number = 0

  @classmethod
  def reset(cls):
    """Method resetting Box to initial values."""
    cls.boxes_number = 0
    return

  def __init__(self,ctn_type='box',source=None):
    """
    Parameters
    ----------
    ctn_type : {'figure','table','note','box'}, optional
      box content type
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      box number
    style : str
      box style
    ctn_type : {'figure','table','note','box'}
      box content type
    ctn_options : str
      box content options
    ctn : str
      box content
    cap_type : {'figure','table','note'}
      box caption type
    cap_options : str
      box caption options
    cap : str
      box caption
    """
    self.number      = 0
    self.ctn_type    = ctn_type
    self.style       = None
    self.ctn_options = None
    self.cap_options = None
    self.cap_type    = None
    self.ctn         = None
    self.cap         = None
    if self.ctn_type == 'box':
      Box.boxes_number += 1
      self.number = Box.boxes_number
    if source:
      self.get(source=source)
    return

  def __str__(self):
    string = []
    string.append('\nStyle: '+str(self.style))
    string.append('\nCaption('+str(self.cap_type)+')[options='+str(self.cap_options)+']: '+str(self.cap))
    string.append('\nContent('+str(self.ctn_type)+')[options='+str(self.ctn_options)+']: '+str(self.ctn))
    return ''.join(string)

  def __get_style(self,source):
    """Method for getting box style data."""
    match = re.search(__restl__,source)
    if match:
      style = match.group('style')
      if style:
        self.style = style.strip()
    return

  def __get_caption(self,source):
    """Method for getting box caption data."""
    match = re.search(__recap__,source)
    if match:
      cap_type = match.group('cap_type')
      if cap_type:
        if cap_type.lower() == 'none':
          self.cap_type = None
        else:
          self.cap_type = cap_type.strip()
      cap_options = match.group('cap_options')
      if cap_options:
        self.cap_options = cap_options.strip()
      self.cap = match.group('cap')
      if self.cap:
        self.cap = self.cap.strip()
    return

  def __get_content(self,source):
    """Method for getting box content data."""
    match = re.search(__rectn__,source)
    if match:
      ctn_type = match.group('ctn_type')
      if ctn_type:
        self.ctn_type = ctn_type.strip()
      ctn_options = match.group('ctn_options')
      if ctn_options:
        self.ctn_options = ctn_options.strip()
      self.ctn = match.group('ctn')
      if self.ctn:
        self.ctn = self.ctn.strip()
    return

  def get(self,source):
    """Method for getting box data."""
    self.__get_style(source)
    self.__get_caption(source)
    self.__get_content(source)
    return

  def caption_txt(self):
    """Method for building caption text.

    Returns
    -------
    str
      caption text
    """
    if self.cap_type and self.cap:
      txt = self.cap_type+' '+str(self.number)+': '+seditor.md_convert(source=self.cap,no_p=True)
    elif self.cap_type:
      txt = self.cap_type
    elif self.cap:
      txt = seditor.md_convert(source=self.cap,no_p=True)
    return txt

  def put_caption(self,doc):
    """Method for inserting caption into doc.

    Parameters
    ----------
    doc : yattag.Doc object
      yattag document where to put caption
    """
    if self.cap or self.cap_type:
      with doc.tag('div',klass='box caption'):
        if self.cap_options:
          doc.attr(style=self.cap_options)
        doc.asis(self.caption_txt())
    return

  def to_html(self):
    """Method for inserting box to the html doc."""
    doc = Doc()
    with doc.tag('div',klass='box'):
      if self.style:
        doc.attr(style=self.style)
      with doc.tag('div',klass='box content'):
        if self.ctn_options:
          doc.attr(style=self.ctn_options)
        doc.asis(self.ctn)
      self.put_caption(doc=doc)
    return doc.getvalue()

def tokenize(source):
  """Method for tokenizing source tagging boxes environments.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  list
    list of tokens whose elements are ['type',source_part]; type is 'box' for box environments and
    'unknown' for anything else
  """
  return generic_tokenize(source=source,re_part=__rebox__,name_part='box')

def parse(source):
  """Method for parsing source substituting boxes with their own html equivalent.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  str
    source string parsed
  """
  protected, obfuscate_source = obfuscate(source = source)
  for match in re.finditer(__rebox__,obfuscate_source):
    box = Box(source=illuminate(source=match.group('env'),protected_contents=protected))
    obfuscate_source = re.sub(__rebox__,box.to_html(),obfuscate_source,1)
  return illuminate(source=obfuscate_source,protected_contents=protected)
